#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_launch_template_info
version_added: 9.0.0
short_description: Gather information about launch templates and versions
description:
  - Gather information about launch templates.
author:
  - Aubin Bikouo (@abikouo)
options:
  launch_template_ids:
    description: The IDs of the launch templates.
    type: list
    elements: str
  filters:
    description:
      - A dict of filters to apply. Each dict item consists of a filter key and a filter value.
      - See U(https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeLaunchTemplates.html) for possible filters.
      - Filter names and values are case sensitive.
    type: dict
    default: {}

extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Gather information about a launch template
  amazon.aws.ec2_launch_template_info:
    launch_template_ids:
      - 'lt-01238c059e3466abc'

- name: Gather information launch template using name
  amazon.aws.ec2_launch_template_info:
    filters:
      launch-template-name: my-test-launch-template
"""

RETURN = r"""
launch_templates:
  description: A list of launch templates.
  returned: always
  type: list
  elements: dict
  contains:
    launch_template_id:
      description: The ID of the launch template.
      type: str
      returned: always
    launch_template_name:
      description: The name of the launch template.
      type: str
      returned: always
    create_time:
      description: The time launch template was created.
      type: str
      returned: always
    created_by:
      description: The principal that created the launch template.
      type: str
      returned: always
    default_version_number:
      description: The version number of the default version of the launch template.
      type: int
      returned: always
    latest_version_number:
      description: The version number of the latest version of the launch template.
      type: int
      returned: always
    tags:
      description: A dictionary of tags assigned to image.
      returned: when AMI is created or already exists
      type: dict
      sample: {
          "Env": "devel",
          "Name": "nat-server"
      }
    versions:
      description: All available versions of the launch template.
      type: list
      elements: dict
      returned: always
      contains:
        launch_template_id:
          description: The ID of the launch template.
          type: str
          returned: always
        launch_template_name:
          description: The name of the launch template.
          type: str
          returned: always
        create_time:
          description: The time the version was created.
          type: str
          returned: always
        created_by:
          description: The principal that created the version.
          type: str
          returned: always
        default_version:
          description: Indicates whether the version is the default version.
          type: bool
          returned: always
        version_number:
          description: The version number.
          type: int
          returned: always
        version_description:
          description: The description for the version.
          type: str
          returned: always
        launch_template_data:
          description: Information about the launch template.
          returned: always
          type: dict
          contains:
            kernel_id:
              description:
                - The ID of the kernel.
              returned: if applicable
              type: str
            image_id:
              description: The ID of the AMI or a Systems Manager parameter.
              type: str
              returned: if applicable
            instance_type:
              description: The instance type.
              type: str
              returned: if applicable
            key_name:
              description: The name of the key pair.
              type: str
              returned: if applicable
            monitoring:
              description: The monitoring for the instance.
              type: dict
              returned: if applicable
              contains:
                enabled:
                  description: Indicates whether detailed monitoring is enabled. Otherwise, basic monitoring is enabled.
                  type: bool
                  returned: always
            placement:
              description: The placement of the instance.
              type: dict
              returned: if applicable
              contains:
                availability_zone:
                  description: The Availability Zone of the instance.
                  type: str
                  returned: if applicable
                affinity:
                  description: The affinity setting for the instance on the Dedicated Host.
                  type: str
                  returned: if applicable
                group_name:
                  description: The name of the placement group for the instance.
                  type: str
                  returned: if applicable
                host_id:
                  description: The ID of the Dedicated Host for the instance.
                  type: str
                  returned: if applicable
                tenancy:
                  description: The tenancy of the instance.
                  type: str
                  returned: if applicable
                host_resource_group_arn:
                  description: The ARN of the host resource group in which to launch the instances.
                  type: str
                  returned: if applicable
                partition_number:
                  description: The number of the partition the instance should launch in.
                  type: int
                  returned: if applicable
                group_id:
                  description: The Group ID of the placement group.
                  type: str
                  returned: if applicable
            ebs_optimized:
              description:
                - Indicates whether the instance is optimized for Amazon EBS I/O.
              type: bool
              returned: always
            iam_instance_profile:
              description:
                - The IAM instance profile.
              type: dict
              returned: if application
              contains:
                arn:
                  description: The Amazon Resource Name (ARN) of the instance profile.
                  type: str
                  returned: always
                name:
                  description: The name of the instance profile.
                  type: str
                  returned: always
            block_device_mappings:
              description: The block device mappings.
              type: list
              elements: dict
              returned: if applicable
              contains:
                device_name:
                  description: The device name.
                  type: str
                  returned: always
                virtual_name:
                  description: The virtual device name.
                  type: str
                  returned: always
                ebs:
                  description: Information about the block device for an EBS volume.
                  type: str
                  returned: if applicable
                  contains:
                    encrypted:
                      description: Indicates whether the EBS volume is encrypted.
                      type: bool
                      returned: always
                    delete_on_termination:
                      description: Indicates whether the EBS volume is deleted on instance termination.
                      type: bool
                      returned: always
                    iops:
                      description: The number of I/O operations per second (IOPS) that the volume supports.
                      type: int
                      returned: always
                    kms_key_id:
                      description: The ARN of the Key Management Service (KMS) CMK used for encryption.
                      type: int
                      returned: always
                    snapshot_id:
                      description: The ID of the snapshot.
                      type: str
                      returned: always
                    volume_size:
                      description: The size of the volume, in GiB.
                      type: int
                      returned: always
                    volume_type:
                      description: The volume type.
                      type: str
                      returned: always
                    throughput:
                      description: The throughput that the volume supports, in MiB/s.
                      type: int
                      returned: always
                no_device:
                  description: To omit the device from the block device mapping, specify an empty string.
                  type: str
            network_interfaces:
              description: The network interfaces.
              type: list
              elements: dict
              returned: if applicable
              contains:
                associate_carrier_ip_address:
                  description: Indicates whether to associate a Carrier IP address with eth0 for a new network interface.
                  type: bool
                  returned: always
                associate_public_ip_address:
                  description: Indicates whether to associate a public IPv4 address with eth0 for a new network interface.
                  type: bool
                  returned: always
                delete_on_termination:
                  description: Indicates whether the network interface is deleted when the instance is terminated.
                  type: bool
                  returned: always
                description:
                  description: A description for the network interface.
                  type: str
                  returned: always
                device_index:
                  description: The device index for the network interface attachment.
                  type: int
                  returned: always
                groups:
                  description: The IDs of one or more security groups.
                  type: list
                  elements: str
                  returned: if applicable
                interface_type:
                  description: The type of network interface.
                  type: str
                  returned: always
                ipv6_address_count:
                  description: The number of IPv6 addresses for the network interface.
                  type: int
                  returned: if applicable
                ipv6_addresses:
                  description: The IPv6 addresses for the network interface.
                  returned: if applicable
                  type: list
                  elements: dict
                  contains:
                    ipv6_address:
                      description: The IPv6 address.
                      type: str
                      returned: always
                    is_primary_ipv6:
                      description: Determines if an IPv6 address associated with a network interface is the primary IPv6 address.
                      type: bool
                      returned: always
                network_interface_id:
                  description: The ID of the network interface.
                  type: str
                  returned: always
                private_ip_address:
                  description: The primary private IPv4 address of the network interface.
                  type: str
                  returned: if applicable
                private_ip_addresses:
                  description: A list of private IPv4 addresses.
                  type: list
                  elements: str
                  returned: if applicable
                  contains:
                    primary:
                      description: Indicates whether the private IPv4 address is the primary private IPv4 address.
                      type: bool
                      returned: always
                    private_ip_address:
                      description: The private IPv4 address.
                      type: bool
                      returned: always
                secondary_private_ip_address_count:
                  description: The number of secondary private IPv4 addresses for the network interface.
                  type: int
                  returned: if applicable
                subnet_id:
                  description: The ID of the subnet for the network interface.
                  type: str
                  returned: always
                network_card_index:
                  description: The index of the network card.
                  type: int
                  returned: if applicable
                ipv4_prefixes:
                  description: A list of IPv4 prefixes assigned to the network interface.
                  type: list
                  elements: dict
                  returned: if applicable
                  contains:
                    ipv4_prefix:
                      description: The IPv4 delegated prefixes assigned to the network interface.
                      type: str
                      returned: always
                ipv4_prefix_count:
                  description: The number of IPv4 prefixes that Amazon Web Services automatically assigned to the network interface.
                  type: int
                  returned: if applicable
                ipv6_prefixes:
                  description: A list of IPv6 prefixes assigned to the network interface.
                  type: list
                  elements: dict
                  returned: if applicable
                  contains:
                    ipv6_prefix:
                      description: The IPv6 delegated prefixes assigned to the network interface.
                      type: str
                      returned: always
                ipv6_prefix_count:
                  description: The number of IPv6 prefixes that Amazon Web Services automatically assigned to the network interface.
                  type: int
                  returned: if applicable
                primary_ipv6:
                  description: The primary IPv6 address of the network interface.
                  type: str
                  returned: if applicable
                ena_srd_specification:
                  description: Contains the ENA Express settings for instances launched from your launch template.
                  type: dict
                  returned: if applicable
                  contains:
                    ena_srd_enabled:
                      description: Indicates whether ENA Express is enabled for the network interface.
                      type: bool
                      returned: always
                    ena_srd_udp_specification:
                      description: Configures ENA Express for UDP network traffic.
                      type: dict
                      returned: always
                      contains:
                        ena_srd_udp_enabled:
                          description: Indicates whether UDP traffic to and from the instance uses ENA Express.
                          type: bool
                          returned: always
                connection_tracking_specification:
                  description:
                    - A security group connection tracking specification that enables you to set the timeout
                      for connection tracking on an Elastic network interface.
                  type: dict
                  returned: if applicable
                  contains:
                    tcp_established_timeout:
                      description: Timeout (in seconds) for idle TCP connections in an established state.
                      type: int
                      returned: always
                    udp_timeout:
                      description:
                        - Timeout (in seconds) for idle UDP flows that have seen traffic only in a single direction
                          or a single request-response transaction.
                      type: int
                      returned: always
                    udp_stream_timeout:
                      description:
                        - Timeout (in seconds) for idle UDP flows classified as streams which have seen more
                          than one request-response transaction.
                      type: int
                      returned: always
            ram_disk_id:
              description: The ID of the RAM disk, if applicable.
              type: str
              returned: if applicable
            disable_api_termination:
              description: If set to true, indicates that the instance cannot be terminated using the Amazon EC2 console, command line tool, or API.
              type: bool
              returned: if applicable
            instance_initiated_shutdown_behavior:
              description: Indicates whether an instance stops or terminates when you initiate shutdown from the instance.
              type: str
              returned: if applicable
            user_data:
              description: The user data for the instance.
              type: str
              returned: if applicable
            tag_specifications:
              description: The tags that are applied to the resources that are created during instance launch.
              type: list
              elements: dict
              returned: if applicable
              contains:
                resource_type:
                  description: The type of resource to tag.
                  type: str
                  returned: always
                tags:
                  description: The tags for the resource.
                  type: list
                  elements: dict
                  contains:
                    key:
                      description: The key of the tag.
                      type: str
                      returned: always
                    value:
                      description: The value of the tag.
                      type: str
                      returned: always
            enclave_options:
              description: Indicates whether the instance is enabled for Amazon Web Services Nitro Enclaves.
              type: dict
              returned: if applicable
              contains:
                enabled:
                  description: If this parameter is set to true, the instance is enabled for Amazon Web Services Nitro Enclaves.
                  type: bool
                  returned: always
            metadata_options:
              description: The metadata options for the instance.
              type: dict
              returned: if applicable
              contains:
                state:
                  description: The state of the metadata option changes.
                  type: str
                  returned: if applicable
                http_tokens:
                  description: Indicates whether IMDSv2 is required.
                  type: str
                  returned: if applicable
                http_put_response_hop_limit:
                  description: The desired HTTP PUT response hop limit for instance metadata requests.
                  type: int
                  returned: if applicable
                http_endpoint:
                  description: Enables or disables the HTTP metadata endpoint on your instances.
                  type: str
                  returned: if applicable
                http_protocol_ipv6:
                  description: Enables or disables the IPv6 endpoint for the instance metadata service.
                  type: str
                  returned: if applicable
                instance_metadata_tags:
                  description: Set to enabled to allow access to instance tags from the instance metadata.
                  type: str
                  returned: if applicable
            cpu_options:
              description: The CPU options for the instance.
              type: dict
              returned: if applicable
              contains:
                core_count:
                  description: The number of CPU cores for the instance.
                  type: int
                  returned: if applicable
                threads_per_core:
                  description: The number of threads per CPU core.
                  type: int
                  returned: if applicable
                amd_sev_snp:
                  description: Indicates whether the instance is enabled for AMD SEV-SNP.
                  type: int
                  returned: if applicable
            security_group_ids:
              description: The security group IDs.
              type: list
              elements: str
              returned: if applicable
            security_groups:
              description: The security group names.
              type: list
              elements: str
              returned: if applicable
"""

from typing import Any
from typing import Dict
from typing import List

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import normalize_boto3_result
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import AnsibleEC2Error
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_launch_template_versions
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_launch_templates
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list


def list_launch_templates(client, module: AnsibleAWSModule) -> List[Dict[str, Any]]:
    try:
        # Describe launch templates
        launch_templates = describe_launch_templates(
            client,
            launch_template_ids=module.params.get("launch_template_ids"),
            filters=ansible_dict_to_boto3_filter_list(module.params.get("filters")),
        )

        # Describe launch templates versions
        for template in launch_templates:
            template["Versions"] = describe_launch_template_versions(
                client, LaunchTemplateId=template["LaunchTemplateId"]
            )

        # format output
        launch_templates = [camel_dict_to_snake_dict(t, ignore_list=["Tags"]) for t in launch_templates]
        for t in launch_templates:
            t["tags"] = boto3_tag_list_to_ansible_dict(t.pop("tags", {}))

        return normalize_boto3_result(launch_templates)

    except AnsibleEC2Error as e:
        module.fail_json_aws_error(e)


def main():
    argument_spec = dict(
        launch_template_ids=dict(type="list", elements="str"),
        filters=dict(default={}, type="dict"),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)
    client = module.client("ec2")

    launch_templates = list_launch_templates(client, module)
    module.exit_json(launch_templates=launch_templates)


if __name__ == "__main__":
    main()



from __future__ import absolute_import, division, print_function

__metaclass__ = type

import pytest

from ansible_base.rbac.models import RoleDefinition


@pytest.mark.django_db
def test_create_new(run_module, admin_user):
    result = run_module(
        'role_definition',
        {
            'name': 'test_view_jt',
            'permissions': ['awx.view_jobtemplate', 'awx.execute_jobtemplate'],
            'content_type': 'awx.jobtemplate',
        },
        admin_user)
    assert result['changed']

    role_definition = RoleDefinition.objects.get(name='test_view_jt')
    assert role_definition
    permission_codenames = [p.codename for p in role_definition.permissions.all()]
    assert set(permission_codenames) == set(['view_jobtemplate', 'execute_jobtemplate'])
    assert role_definition.content_type.model == 'jobtemplate'


@pytest.mark.django_db
def test_update_existing(run_module, admin_user):
    result = run_module(
        'role_definition',
        {
            'name': 'test_view_jt',
            'permissions': ['awx.view_jobtemplate'],
            'content_type': 'awx.jobtemplate',
        },
        admin_user)

    assert result['changed']

    role_definition = RoleDefinition.objects.get(name='test_view_jt')
    permission_codenames = [p.codename for p in role_definition.permissions.all()]
    assert set(permission_codenames) == set(['view_jobtemplate'])
    assert role_definition.content_type.model == 'jobtemplate'

    result = run_module(
        'role_definition',
        {
            'name': 'test_view_jt',
            'permissions': ['awx.view_jobtemplate', 'awx.execute_jobtemplate'],
            'content_type': 'awx.jobtemplate',
        },
        admin_user)

    assert result['changed']

    role_definition.refresh_from_db()
    permission_codenames = [p.codename for p in role_definition.permissions.all()]
    assert set(permission_codenames) == set(['view_jobtemplate', 'execute_jobtemplate'])
    assert role_definition.content_type.model == 'jobtemplate'


@pytest.mark.django_db
def test_delete_existing(run_module, admin_user):
    result = run_module(
        'role_definition',
        {
            'name': 'test_view_jt',
            'permissions': ['awx.view_jobtemplate', 'awx.execute_jobtemplate'],
            'content_type': 'awx.jobtemplate',
        },
        admin_user)

    assert result['changed']

    role_definition = RoleDefinition.objects.get(name='test_view_jt')
    assert role_definition

    result = run_module(
        'role_definition',
        {
            'name': 'test_view_jt',
            'permissions': ['awx.view_jobtemplate', 'awx.execute_jobtemplate'],
            'content_type': 'awx.jobtemplate',
            'state': 'absent',
        },
        admin_user)

    assert result['changed']

    with pytest.raises(RoleDefinition.DoesNotExist):
        role_definition.refresh_from_db()


@pytest.mark.django_db
def test_idempotence(run_module, admin_user):
    result = run_module(
        'role_definition',
        {
            'name': 'test_view_jt',
            'permissions': ['awx.view_jobtemplate', 'awx.execute_jobtemplate'],
            'content_type': 'awx.jobtemplate',
        },
        admin_user)

    assert result['changed']

    result = run_module(
        'role_definition',
        {
            'name': 'test_view_jt',
            'permissions': ['awx.view_jobtemplate', 'awx.execute_jobtemplate'],
            'content_type': 'awx.jobtemplate',
        },
        admin_user)

    assert not result['changed']

    role_definition = RoleDefinition.objects.get(name='test_view_jt')
    permission_codenames = [p.codename for p in role_definition.permissions.all()]
    assert set(permission_codenames) == set(['view_jobtemplate', 'execute_jobtemplate'])



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: health_score_definitions_count_v1_info
short_description: Information module for Health Score Definitions Count V1
description:
- Get all Health Score Definitions Count V1.
- >
   Get the count of health score definitions based on provided filters. Supported filters are id, name and overall
   health include status. For detailed information about the usage of the API, please refer to the Open API
   specification document - https //github.com/cisco-en-programmability/catalyst-center-api-
   specs/blob/main/Assurance/CE_Cat_Center_Org-issueAndHealthDefinitions-1.0.0-resolved.yaml.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  deviceType:
    description:
    - >
      DeviceType query parameter. These are the device families supported for health score definitions. If no
      input is made on device family, all device families are considered.
    type: str
  id:
    description:
    - >
      Id query parameter. The definition identifier. Examples id=015d9cba-4f53-4087-8317-7e49e5ffef46 (single
      entity id request) id=015d9cba-4f53-4087-8317-7e49e5ffef46&id=015d9cba-4f53-4087-8317-7e49e5ffef47 (multiple
      ids in the query param).
    type: str
  includeForOverallHealth:
    description:
    - >
      IncludeForOverallHealth query parameter. The inclusion status of the issue definition, either true or false.
      True indicates that particular health metric is included in overall health computation, otherwise false. By
      default it's set to true.
    type: bool
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices GetTheCountOfHealthScoreDefinitionsBasedOnProvidedFiltersV1
  description: Complete reference of the GetTheCountOfHealthScoreDefinitionsBasedOnProvidedFiltersV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-the-count-of-health-score-definitions-based-on-provided-filters
notes:
  - SDK Method used are
    devices.Devices.get_the_count_of_health_score_definitions_based_on_provided_filters_v1,

  - Paths used are
    get /intent/api/v1/healthScoreDefinitions/count,

"""

EXAMPLES = r"""
- name: Get all Health Score Definitions Count V1
  cisco.dnac.health_score_definitions_count_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    deviceType: string
    id: string
    includeForOverallHealth: True
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "count": 0
      },
      "version": "string"
    }
"""



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The iosxr snmp_server fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.snmp_server.snmp_server import (
    Snmp_serverArgs,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.rm_templates.snmp_server import (
    Snmp_serverTemplate,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.utils.utils import (
    flatten_config,
)


class Snmp_serverFacts(object):
    """The iosxr snmp_server facts class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Snmp_serverArgs.argument_spec

    def get_config(self, connection):
        return connection.get("show running-config snmp-server")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for Snmp_server network resource

        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf

        :rtype: dictionary
        :returns: facts
        """
        facts = {}
        objs = []

        if not data:
            data = self.get_config(connection)

        flatten_context_list = [
            "snmp-server vrf",
            "snmp-server mib bulkstat schema",
            "snmp-server mib bulkstat transfer-id",
            "snmp-server correlator rule",
            "snmp-server interface",
            "snmp-server correlator rule",
            "snmp-server correlator ruleset",
        ]

        for x in flatten_context_list:
            data = flatten_config(data, x)
        # parse native config using the Snmp_server template
        snmp_server_parser = Snmp_serverTemplate(
            lines=data.splitlines(),
            module=self._module,
        )
        objs = snmp_server_parser.parse()

        dict_to_list = [
            "context",
            "mib_object_lists",
            "mib_schema",
            "mib_bulkstat_transfer_ids",
            "vrfs",
            "interfaces",
        ]
        for i in dict_to_list:
            if i in objs:
                objs[i] = list(objs[i].values())
                if i == "vrfs":
                    for j in objs[i]:
                        j["hosts"].remove({})
                        j["context"] = list(j["context"].values())

        ansible_facts["ansible_network_resources"].pop("snmp_server", None)

        params = utils.remove_empties(
            snmp_server_parser.validate_config(
                self.argument_spec,
                {"config": objs},
                redact=True,
            ),
        )
        facts["snmp_server"] = params.get("config", {})
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: aws_region_info
short_description: Gather information about AWS regions
version_added: 1.0.0
version_added_collection: community.aws
description:
  - Gather information about AWS regions.
author:
  - 'Henrique Rodrigues (@Sodki)'
options:
  filters:
    description:
      - A dict of filters to apply.
      - Each dict item consists of a filter key and a filter value.
      - See U(https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeRegions.html) for possible filters.
      - Filter names and values are case sensitive.
      - You can use underscores instead of dashes (-) in the filter keys.
      - Filter keys with underscores will take precedence in case of conflict.
    default: {}
    type: dict
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

# Gather information about all regions
- amazon.aws.aws_region_info:

# Gather information about a single region
- amazon.aws.aws_region_info:
    filters:
      region-name: eu-west-1
"""

RETURN = r"""
regions:
    returned: on success
    description:
        - Information about the regions that match the provided filters. Retruns information about all the regions if no filters specified.
        - Each element consists of a dict with all the information related to that region.
    type: list
    elements: dict
    sample: "[{
        'endpoint': 'ec2.us-west-1.amazonaws.com',
        'region_name': 'us-west-1'
    }]"
    contains:
        endpoint:
            description: The Region service endpoint.
            type: str
            sample: "ec2.us-east-2.amazonaws.com"
        opt_in_status:
            description: The Region opt-in status.
            type: str
            sample: "opt-in-not-required"
        region_name:
            description: The name of the Region.
            type: str
            sample: "us-east-2"
"""

try:
    from botocore.exceptions import BotoCoreError
    from botocore.exceptions import ClientError
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_regions
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.transformation import sanitize_filters_to_boto3_filter_list


def main():
    argument_spec = dict(
        filters=dict(default={}, type="dict"),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    connection = module.client("ec2")

    # Sanitize filters
    sanitized_filters = sanitize_filters_to_boto3_filter_list(module.params.get("filters"))
    try:
        regions = describe_regions(connection, Filters=sanitized_filters)
    except (BotoCoreError, ClientError) as e:
        module.fail_json_aws(e, msg="Unable to describe regions.")

    module.exit_json(regions=[camel_dict_to_snake_dict(r) for r in regions])


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: authentication_policy_servers
short_description: Resource module for Authentication Policy Servers
description:
- This module represents an alias of the module authentication_policy_servers_v1
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  accountingPort:
    description: Accounting port of RADIUS server (readonly). The range is from 1 to
      65535. E.g. 1813.
    type: int
  authenticationPort:
    description: Authentication port of RADIUS server (readonly). The range is from
      1 to 65535. E.g. 1812.
    type: int
  ciscoIseDtos:
    description: Authentication Policy Servers's ciscoIseDtos.
    elements: dict
    suboptions:
      description:
        description: Description about the Cisco ISE server.
        type: str
      fqdn:
        description: Fully-qualified domain name of the Cisco ISE server (readonly).
          E.g. Xi-62.my.com.
        type: str
      ipAddress:
        description: IP Address of the Cisco ISE Server (readonly).
        type: str
      password:
        description: Password of the Cisco ISE server.
        type: str
      sshkey:
        description: SSH key of the Cisco ISE server.
        type: str
      subscriberName:
        description: Subscriber name of the Cisco ISE server (readonly). E.g. Pxgrid_client_1662589467.
        type: str
      userName:
        description: User name of the Cisco ISE server.
        type: str
    type: list
  encryptionKey:
    description: Encryption key used to encrypt shared secret (readonly).
    type: str
  encryptionScheme:
    description: Type of encryption scheme for additional security (readonly).
    type: str
  externalCiscoIseIpAddrDtos:
    description: Authentication Policy Servers's externalCiscoIseIpAddrDtos.
    elements: dict
    suboptions:
      externalCiscoIseIpAddresses:
        description: Authentication Policy Servers's externalCiscoIseIpAddresses.
        elements: dict
        suboptions:
          externalIpAddress:
            description: External IP Address.
            type: str
        type: list
      type:
        description: Type.
        type: str
    type: list
  id:
    description: Id path parameter. Authentication and Policy Server Identifier. Use
      'Get Authentication and Policy Servers' intent API to find the identifier.
    type: str
  ipAddress:
    description: IP address of authentication and policy server (readonly).
    type: str
  isIseEnabled:
    description: Value true for Cisco ISE Server (readonly). Default value is false.
    type: bool
  messageKey:
    description: Message key used to encrypt shared secret (readonly).
    type: str
  port:
    description: Port of TACACS server (readonly). The range is from 1 to 65535.
    type: int
  protocol:
    description: Type of protocol for authentication and policy server. If already saved
      with RADIUS, can update to RADIUS_TACACS. If already saved with TACACS, can update
      to RADIUS_TACACS.
    type: str
  pxgridEnabled:
    description: Value true for enable, false for disable. Default value is true.
    type: bool
  retries:
    description: Number of communication retries between devices and authentication
      and policy server. The range is from 1 to 3.
    type: str
  role:
    description: Role of authentication and policy server (readonly). E.g. Primary,
      secondary.
    type: str
  sharedSecret:
    description: Shared secret between devices and authentication and policy server
      (readonly).
    type: str
  timeoutSeconds:
    description: Number of seconds before timing out between devices and authentication
      and policy server. The range is from 2 to 20.
    type: str
  useDnacCertForPxgrid:
    description: Value true to use DNAC certificate for Pxgrid. Default value is false.
    type: bool
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for System Settings AddAuthenticationAndPolicyServerAccessConfigurationV1
  description: Complete reference of the AddAuthenticationAndPolicyServerAccessConfigurationV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!add-authentication-and-policy-server-access-configuration
- name: Cisco DNA Center documentation for System Settings DeleteAuthenticationAndPolicyServerAccessConfigurationV1
  description: Complete reference of the DeleteAuthenticationAndPolicyServerAccessConfigurationV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-authentication-and-policy-server-access-configuration
- name: Cisco DNA Center documentation for System Settings EditAuthenticationAndPolicyServerAccessConfigurationV1
  description: Complete reference of the EditAuthenticationAndPolicyServerAccessConfigurationV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!edit-authentication-and-policy-server-access-configuration
notes:
  - SDK Method used are
    system_settings.SystemSettings.add_authentication_and_policy_server_access_configuration_v1,
    system_settings.SystemSettings.delete_authentication_and_policy_server_access_configuration_v1,
    system_settings.SystemSettings.edit_authentication_and_policy_server_access_configuration_v1,

  - Paths used are
    post /dna/intent/api/v1/authentication-policy-servers,
    delete /dna/intent/api/v1/authentication-policy-servers/{id},
    put /dna/intent/api/v1/authentication-policy-servers/{id},
  - It should be noted that this module is an alias of authentication_policy_servers_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.authentication_policy_servers:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    accountingPort: 0
    authenticationPort: 0
    ciscoIseDtos:
    - description: string
      fqdn: string
      ipAddress: string
      password: string
      sshkey: string
      subscriberName: string
      userName: string
    encryptionKey: string
    encryptionScheme: string
    externalCiscoIseIpAddrDtos:
    - externalCiscoIseIpAddresses:
      - externalIpAddress: string
      type: string
    ipAddress: string
    isIseEnabled: true
    messageKey: string
    port: 0
    protocol: string
    pxgridEnabled: true
    retries: string
    role: string
    sharedSecret: string
    timeoutSeconds: string
    useDnacCertForPxgrid: true

- name: Delete by id
  cisco.dnac.authentication_policy_servers:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    id: string

- name: Update by id
  cisco.dnac.authentication_policy_servers:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    accountingPort: 0
    authenticationPort: 0
    ciscoIseDtos:
    - description: string
      fqdn: string
      ipAddress: string
      password: string
      sshkey: string
      subscriberName: string
      userName: string
    encryptionKey: string
    encryptionScheme: string
    externalCiscoIseIpAddrDtos:
    - externalCiscoIseIpAddresses:
      - externalIpAddress: string
      type: string
    id: string
    ipAddress: string
    isIseEnabled: true
    messageKey: string
    port: 0
    protocol: string
    pxgridEnabled: true
    retries: string
    role: string
    sharedSecret: string
    timeoutSeconds: string
    useDnacCertForPxgrid: true

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "taskId": "string",
      "url": "string"
    }
"""



from __future__ import (absolute_import, division, print_function)
__metaclass__ = type
from ansible.plugins.action import ActionBase
try:
    from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
        AnsibleArgSpecValidator,
    )
except ImportError:
    ANSIBLE_UTILS_IS_INSTALLED = False
else:
    ANSIBLE_UTILS_IS_INSTALLED = True
from ansible.errors import AnsibleActionFail
from ansible_collections.cisco.ise.plugins.plugin_utils.personas_utils import Node
from ansible_collections.cisco.ise.plugins.plugin_utils.ise import (
    ise_compare_equality,
)

argument_spec = dict(
    ip=dict(type="str", required=True),
    username=dict(type="str", required=True),
    password=dict(type="str", required=True),
    hostname=dict(type="str", required=True),
    roles=dict(type="list", required=True),
    services=dict(type="list", required=True),
    ise_verify=dict(type="bool", default=True),
    ise_version=dict(type="str", default="3.0.0"),
    ise_wait_on_rate_limit=dict(type="bool", default=True),  # TODO: verify what the true default value should be
)

required_if = []
required_one_of = []
mutually_exclusive = []
required_together = []


class NodeDeployment(object):
    def requires_update(self, current_obj, requested_obj):
        obj_params = [
            ("roles", "roles"),
            ("services", "services"),
        ]
        # Method 1. Params present in request (Ansible) obj are the same as the current (ISE) params
        # If any does not have eq params, it requires update
        return any(not ise_compare_equality(current_obj.get(ise_param),
                                            requested_obj.get(ansible_param))
                   for (ise_param, ansible_param) in obj_params)


class ActionModule(ActionBase):
    def __init__(self, *args, **kwargs):
        if not ANSIBLE_UTILS_IS_INSTALLED:
            raise AnsibleActionFail("ansible.utils is not installed. Execute 'ansible-galaxy collection install ansible.utils'")
        super(ActionModule, self).__init__(*args, **kwargs)
        self._supports_async = False
        self._supports_check_mode = False
        self._result = None

    # Checks the supplied parameters against the argument spec for this module
    def _check_argspec(self):
        aav = AnsibleArgSpecValidator(
            data=self._task.args,
            schema=dict(argument_spec=argument_spec),
            schema_format="argspec",
            schema_conditionals=dict(
                required_if=required_if,
                required_one_of=required_one_of,
                mutually_exclusive=mutually_exclusive,
                required_together=required_together,
            ),
            name=self._task.action,
        )
        valid, errors, self._task.args = aav.validate()
        if not valid:
            raise AnsibleActionFail(errors)

    def run(self, tmp=None, task_vars=None):
        self._task.diff = False
        self._result = super(ActionModule, self).run(tmp, task_vars)
        self._result["changed"] = False
        self._check_argspec()
        obj = NodeDeployment()
        request_obj = dict(ip=self._task.args.get("ip"),
                           username=self._task.args.get("username"),
                           password=self._task.args.get("password"),
                           hostname=self._task.args.get("hostname"),
                           roles=self._task.args.get("roles"),
                           services=self._task.args.get("services"),
                           )
        node = Node(request_obj)
        prev_obj = False
        result = dict(changed=False, result="")
        response = None
        if not node.app_server_is_running():
            raise AnsibleActionFail("Couldn't connect, the node might be still initializing, try again in a few minutes. Error received: 502")
        try:
            prev_obj = node.get_roles_services()
        except Exception as e:
            raise AnsibleActionFail(e)
        if prev_obj:
            if obj.requires_update(prev_obj, request_obj):
                try:
                    node.update_roles_services()
                    response = node.get_roles_services()
                    result["changed"] = True
                    result["result"] = "Object updated"
                except Exception as e:
                    raise AnsibleActionFail("The node might be still initializing. Error received: {e}".format(e=e))
            else:
                response = prev_obj
                result["result"] = "Object already present"
        self._result.update(dict(ise_response=response))
        self._result.update(result)
        return self._result



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The cisco.ios prefix_lists fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.prefix_lists.prefix_lists import (
    Prefix_listsArgs,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.rm_templates.prefix_lists import (
    Prefix_listsTemplate,
)


class Prefix_listsFacts(object):
    """The cisco.ios prefix_lists facts class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Prefix_listsArgs.argument_spec

    def get_prefix_list_data(self, connection):
        return connection.get("show running-config | section ^ip prefix-list|^ipv6 prefix-list")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for Prefix_lists network resource

        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf

        :rtype: dictionary
        :returns: facts
        """
        facts = {}
        objs = []

        if not data:
            data = self.get_prefix_list_data(connection)

        # parse native config using the Prefix_lists template
        prefix_lists_parser = Prefix_listsTemplate(lines=data.splitlines())
        objs = prefix_lists_parser.parse()

        final_objs = []

        _prefix_list = {"ipv4": [], "ipv6": []}
        if objs:
            for prefixes in list(objs.values()):
                _afi = prefixes.pop("afi")
                _prefix_list[_afi].append(
                    prefixes,
                )
            if _prefix_list.get("ipv4"):
                final_objs.append({"afi": "ipv4", "prefix_lists": _prefix_list.pop("ipv4")})
            if _prefix_list.get("ipv6"):
                final_objs.append({"afi": "ipv6", "prefix_lists": _prefix_list.pop("ipv6")})

            ansible_facts["ansible_network_resources"].pop("prefix_lists", None)

        params = utils.remove_empties(
            utils.validate_config(self.argument_spec, {"config": final_objs}),
        )

        facts["prefix_lists"] = params.get("config", [])
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts



#!/usr/bin/python
#
# Copyright (c) 2024 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_localnetworkgateway_info

version_added: "2.4.0"

short_description: Gets or list the specified local network gateway in a resource group

description:
    - Gets or list the specified local network gateway in a resource group.

options:
    resource_group:
        description:
            - The local network gateway's resource group.
        type: str
        required: true
    name:
        description:
            - The name of the local network gateway.
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - xuzhang3 (@xuzhang3)
    - Fred Sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Gets the specified local network gateway in a resource group
  azure_rm_localnetworkgateway_info:
    resource_group: "{{ resource_group }}"
    name: "{{ local_networkgateway_name }}"

- name: Gets all the local network gateways in a resource group
  azure_rm_localnetworkgateway_info:
    resource_group: "{{ resource_group }}"

- name: Gets all the local network gateways in a resource group and filter by tags
  azure_rm_localnetworkgateway_info:
    resource_group: "{{ resource_group }}"
    tags:
      - foo
'''

RETURN = '''
state:
    description:
        - Current state of the Azure Local Network Gateway resource.
    returned: always
    type: complex
    contains:
        id:
            description:
                - The resource ID.
            type: str
            returned: always
            sample: "/subscriptions/xxxx-xxxx/resourceGroups/testRG/providers/Microsoft.Network/localNetworkGateways/testgateway"
        bgp_settings:
            description:
                - Local network gateway's BGP speaker settings.
            type: complex
            contains:
                asn:
                    description:
                        - The BGP speaker's ASN.
                    type: int
                    returned: always
                    sample: 10
                bgp_peering_address:
                    description:
                        - The BGP peering address and BGP identifier of this BGP speaker.
                    type: str
                    returned: always
                    sample: 10.0.0.3
                peer_weight:
                    description:
                        - The weight added to routes learned from this BGP speaker.
                    type: int
                    returned: always
                    sample: 0
        fqdn:
            description:
                - FQDN of local network gateway.
            type: str
            returned: always
            sample: testfqdn.com
        gateway_ip_address:
            description:
                - IP address of local network gateway.
            type: str
            returned: always
            sample: 10.1.1.1
        etag:
            description:
                - A unique read-only string that changes whenever the resource is updated.
            type: str
            returned: always
            sample: b5a32693-2e75-49e0-9137-ded19db658d6
        local_network_address_space:
            description:
                - Local network site address space.
            type: complex
            contains:
                address_prefixes:
                    description:
                        - A list of address blocks reserved for this virtual network in CIDR notation.
                    type: list
                    returned: always
                    sample: ["10.0.0.0/24", "20.0.0.0/24"]
        location:
            description:
                - The resource location.
            type: str
            returned: always
            sample: eastus
        name:
            description:
                - The resource name.
            type: str
            returned: always
            sample: testgateway
        provisioning_state:
            description:
                - The provisioning state of the local network gateway resource.
            type: str
            returned: always
            sample: Succeeded
        tags:
            description:
                - The resource tags.
            type: str
            returned: always
            sample: {'key1': 'value1', 'key2': 'value2'}
        type:
            description:
                - The resource type.
            type: str
            returned: always
            sample: Microsoft.Network/localNetworkGateways
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import HttpResponseError
except Exception:
    # handled in azure_rm_common
    pass


class AzureRMNetworkGateWayInfo(AzureRMModuleBase):
    """Utility class to get Azure Kubernetes Service Credentials facts"""

    def __init__(self):

        self.module_args = dict(
            name=dict(type='str'),
            resource_group=dict(type='str', required=True),
            tags=dict(type='list', elements='str'),
        )

        self.name = None
        self.tags = None

        self.results = dict(
            changed=False,
            state=[],
        )

        super(AzureRMNetworkGateWayInfo, self).__init__(derived_arg_spec=self.module_args,
                                                        supports_check_mode=True,
                                                        supports_tags=False,
                                                        facts_module=True)

    def exec_module(self, **kwargs):

        for key in self.module_args:
            setattr(self, key, kwargs[key])

        if self.name is not None:
            self.results['state'] = self.get_local_network_gateway()
        else:
            self.results['state'] = self.list_local_network_gateway()

        return self.results

    def get_local_network_gateway(self):
        """Gets the specified local network gateway in a resource group"""
        response = None

        try:
            response = self.network_client.local_network_gateways.get(self.resource_group, self.name)
        except HttpResponseError as ec:
            self.log("Gets the specified local network gateway in a resource group Failed, Exception as {0}".format(ec))
        if response and self.has_tags(response.tags, self.tags):
            return [self.format_response(response)]
        else:
            return []

    def list_local_network_gateway(self):
        """Gets all the local network gateways in a resource group"""
        response = None

        try:
            response = self.network_client.local_network_gateways.list(self.resource_group)
        except HttpResponseError as ec:
            self.log("Gets all the local network gateways in a resource group Failed, Exception as {0}".format(ec))

        if response:
            results = []
            for item in response:
                if self.has_tags(item.tags, self.tags):
                    results.append(self.format_response(item))
            return results
        else:
            return []

    def format_response(self, item):
        result = dict(
            id=item.id,
            name=item.name,
            location=item.location,
            type=item.type,
            tags=item.tags,
            etag=item.etag,
            local_network_address_space=dict(),
            gateway_ip_address=item.gateway_ip_address,
            fqdn=item.fqdn,
            provisioning_state=item.provisioning_state,
            bgp_settings=dict(),
        )

        if item.local_network_address_space is not None:
            result['local_network_address_space']['address_prefixes'] = item.local_network_address_space.address_prefixes
        if item.bgp_settings is not None:
            result['bgp_settings']['asn'] = item.bgp_settings.asn
            result['bgp_settings']['bgp_peering_address'] = item.bgp_settings.bgp_peering_address
            result['bgp_settings']['peer_weight'] = item.bgp_settings.peer_weight
        return result


def main():
    """Main module execution code path"""

    AzureRMNetworkGateWayInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# coding: utf-8 -*-

# (c) 2020, Bianca Henderson <bianca@redhat.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1', 'status': ['preview'], 'supported_by': 'community'}


DOCUMENTATION = '''
---
module: inventory_source_update
author: "Bianca Henderson (@beeankha)"
short_description: Update inventory source(s).
description:
    - Update Automation Platform Controller inventory source(s). See
      U(https://www.ansible.com/tower) for an overview.
options:
    name:
      description:
        - The name or id of the inventory source to update.
      required: True
      type: str
      aliases:
        - inventory_source
    inventory:
      description:
        - Name or id of the inventory that contains the inventory source(s) to update.
      required: True
      type: str
    organization:
      description:
        - Name, ID, or named URL of the inventory source's inventory's organization.
      type: str
    wait:
      description:
        - Wait for the job to complete.
      default: False
      type: bool
    interval:
      description:
        - The interval to request an update from the controller.
      required: False
      default: 2
      type: float
    timeout:
      description:
        - If waiting for the job to complete this will abort after this
          amount of seconds
      type: int
extends_documentation_fragment: awx.awx.auth
'''

EXAMPLES = '''
- name: Update a single inventory source
  inventory_source_update:
    name: "Example Inventory Source"
    inventory: "My Inventory"
    organization: Default

- name: Update all inventory sources
  inventory_source_update:
    name: "{{ item }}"
    inventory: "My Other Inventory"
  loop: "{{ query('awx.awx.controller_api', 'inventory_sources', query_params={ 'inventory': 30 }, return_ids=True ) }}"
'''

RETURN = '''
id:
    description: id of the inventory update
    returned: success
    type: int
    sample: 86
status:
    description: status of the inventory update
    returned: success
    type: str
    sample: pending
'''

from ..module_utils.controller_api import ControllerAPIModule


def main():
    # Any additional arguments that are not fields of the item can be added here
    argument_spec = dict(
        name=dict(required=True, aliases=['inventory_source']),
        inventory=dict(required=True),
        organization=dict(),
        wait=dict(default=False, type='bool'),
        interval=dict(default=2.0, type='float'),
        timeout=dict(type='int'),
    )

    # Create a module for ourselves
    module = ControllerAPIModule(argument_spec=argument_spec)

    # Extract our parameters
    name = module.params.get('name')
    inventory = module.params.get('inventory')
    organization = module.params.get('organization')
    wait = module.params.get('wait')
    interval = module.params.get('interval')
    timeout = module.params.get('timeout')

    lookup_data = {}
    if organization:
        lookup_data['organization'] = module.resolve_name_to_id('organizations', organization)
    inventory_object = module.get_one('inventories', name_or_id=inventory, data=lookup_data)

    if not inventory_object:
        module.fail_json(msg='The specified inventory, {0}, was not found.'.format(lookup_data))

    inventory_source_object = module.get_one('inventory_sources', name_or_id=name, data={'inventory': inventory_object['id']})

    if not inventory_source_object:
        module.fail_json(msg='The specified inventory source was not found.')

    # Sync the inventory source(s)
    inventory_source_update_results = module.post_endpoint(inventory_source_object['related']['update'])

    if inventory_source_update_results['status_code'] != 202:
        module.fail_json(msg="Failed to update inventory source, see response for details", response=inventory_source_update_results)

    module.json_output['changed'] = True
    module.json_output['id'] = inventory_source_update_results['json']['id']
    module.json_output['status'] = inventory_source_update_results['json']['status']

    if not wait:
        module.exit_json(**module.json_output)

    # Invoke wait function
    module.wait_on_url(
        url=inventory_source_update_results['json']['url'], object_name=inventory_object, object_type='inventory_update', timeout=timeout, interval=interval
    )

    module.exit_json(**module.json_output)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


class AnsibleDNACException(Exception):
    """Base class for all Ansible DNAC package exceptions."""
    pass


class InconsistentParameters(AnsibleDNACException):
    """Provided parameters are not consistent."""
    pass



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: sites_wireless_settings_ssids_info
short_description: Information module for Sites Wireless Settings Ssids Info
description:
- This module represents an alias of the module sites_wireless_settings_ssids_v1_info
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  siteId:
    description:
    - SiteId path parameter. Site UUID.
    type: str
  limit:
    description:
    - Limit query parameter.
    type: float
  offset:
    description:
    - Offset query parameter.
    type: float
  id:
    description:
    - Id path parameter. SSID ID.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Wireless GetSSIDByIDV1
  description: Complete reference of the GetSSIDByIDV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-ssid-by-id
- name: Cisco DNA Center documentation for Wireless GetSSIDBySiteV1
  description: Complete reference of the GetSSIDBySiteV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-ssid-by-site
notes:
  - SDK Method used are
    wireless.Wireless.get_ssid_by_id_v1,
    wireless.Wireless.get_ssid_by_site_v1,

  - Paths used are
    get /dna/intent/api/v1/sites/{siteId}/wirelessSettings/ssids,
    get /dna/intent/api/v1/sites/{siteId}/wirelessSettings/ssids/{id},
  - It should be noted that this module is an alias of sites_wireless_settings_ssids_v1_info

"""

EXAMPLES = r"""
- name: Get all Sites Wireless Settings Ssids Info
  cisco.dnac.sites_wireless_settings_ssids_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    limit: 0
    offset: 0
    siteId: string
  register: result

- name: Get Sites Wireless Settings Ssids Info by id
  cisco.dnac.sites_wireless_settings_ssids_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    siteId: string
    id: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "ssid": "string",
        "authType": "string",
        "passphrase": "string",
        "isFastLaneEnabled": true,
        "isMacFilteringEnabled": true,
        "ssidRadioType": "string",
        "isBroadcastSSID": true,
        "fastTransition": "string",
        "sessionTimeOutEnable": true,
        "sessionTimeOut": 0,
        "clientExclusionEnable": true,
        "clientExclusionTimeout": 0,
        "basicServiceSetMaxIdleEnable": true,
        "basicServiceSetClientIdleTimeout": 0,
        "directedMulticastServiceEnable": true,
        "neighborListEnable": true,
        "managementFrameProtectionClientprotection": "string",
        "nasOptions": [
          "string"
        ],
        "profileName": "string",
        "policyProfileName": "string",
        "aaaOverride": true,
        "coverageHoleDetectionEnable": true,
        "protectedManagementFrame": "string",
        "multiPSKSettings": [
          {
            "priority": 0,
            "passphraseType": "string",
            "passphrase": "string"
          }
        ],
        "clientRateLimit": 0,
        "rsnCipherSuiteGcmp256": true,
        "rsnCipherSuiteCcmp256": true,
        "rsnCipherSuiteGcmp128": true,
        "rsnCipherSuiteCcmp128": true,
        "ghz6PolicyClientSteering": true,
        "isAuthKey8021x": true,
        "isAuthKey8021xPlusFT": true,
        "isAuthKey8021x_SHA256": true,
        "isAuthKeySae": true,
        "isAuthKeySaePlusFT": true,
        "isAuthKeyPSK": true,
        "isAuthKeyPSKPlusFT": true,
        "isAuthKeyOWE": true,
        "isAuthKeyEasyPSK": true,
        "isAuthKeyPSKSHA256": true,
        "openSsid": "string",
        "isCustomNasIdOptions": true,
        "wlanBandSelectEnable": true,
        "isEnabled": true,
        "authServers": [
          "string"
        ],
        "acctServers": [
          "string"
        ],
        "egressQos": "string",
        "ingressQos": "string",
        "inheritedSiteId": "string",
        "inheritedSiteName": "string",
        "wlanType": "string",
        "l3AuthType": "string",
        "authServer": "string",
        "externalAuthIpAddress": "string",
        "webPassthrough": true,
        "sleepingClientEnable": true,
        "sleepingClientTimeout": 0,
        "aclName": "string",
        "isPosturingEnabled": true,
        "isAuthKeySuiteB1x": true,
        "isAuthKeySuiteB1921x": true,
        "isAuthKeySaeExt": true,
        "isAuthKeySaeExtPlusFT": true,
        "isApBeaconProtectionEnabled": true,
        "ghz24Policy": "string",
        "cckmTsfTolerance": 0,
        "isCckmEnabled": true,
        "isHex": true,
        "isSensorPnp": true,
        "id": "string",
        "isRandomMacFilterEnabled": true,
        "fastTransitionOverTheDistributedSystemEnable": true
      },
      "version": "string"
    }
"""



# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# It would be nice to be able to use autoscaling.XYZ, but we're bound by Ansible's "empty-init"
# policy: https://docs.ansible.com/ansible-core/devel/dev_guide/testing/sanity/empty-init.html


from __future__ import annotations

import typing

# Not intended for general re-use / re-import
from ._autoscaling import common as _common
from ._autoscaling import groups as _groups
from ._autoscaling import instances as _instances
from ._autoscaling import transformations as _transformations
from ._autoscaling import waiters as _waiters
from .retries import AWSRetry

if typing.TYPE_CHECKING:
    from typing import Any
    from typing import Dict
    from typing import List
    from typing import Optional

    from .retries import RetryingBotoClientWrapper
    from .transformation import AnsibleAWSResourceList
    from .transformation import BotoResourceList

# Intended for general use / re-import
AnsibleAutoScalingError = _common.AnsibleAutoScalingError
AutoScalingErrorHandler = _common.AutoScalingErrorHandler
WAITER_MAP = _waiters.WAITER_MAP


def get_autoscaling_groups(
    client: RetryingBotoClientWrapper, group_names: Optional[List[str]] = None
) -> AnsibleAWSResourceList:
    groups = _groups.describe_auto_scaling_groups(client, group_names)
    return _transformations.normalize_autoscaling_groups(groups)


def _get_autoscaling_instances(
    client: RetryingBotoClientWrapper, instance_ids: Optional[List[str]] = None, group_name: Optional[str] = None
) -> BotoResourceList:
    if group_name:
        try:
            groups = _groups.describe_auto_scaling_groups(client, [group_name])
            return groups[0]["Instances"]
        except (KeyError, IndexError):
            return None
    return _instances.describe_auto_scaling_instances(client, instance_ids)


def get_autoscaling_instances(
    client: RetryingBotoClientWrapper, instance_ids: Optional[List[str]] = None, group_name: Optional[str] = None
) -> AnsibleAWSResourceList:
    instances = _get_autoscaling_instances(client, instance_ids=instance_ids, group_name=group_name)
    return _transformations.normalize_autoscaling_instances(instances, group_name=group_name)


def get_autoscaling_waiter(client: RetryingBotoClientWrapper, waiter_name: str) -> Any:
    return _waiters.waiter_factory.get_waiter(client, waiter_name)


# ====================================
# TODO Move these about and refactor
# ====================================


@AutoScalingErrorHandler.list_error_handler("describe InstanceRefreshes", {})
@AWSRetry.jittered_backoff()
def describe_instance_refreshes(
    client,
    auto_scaling_group_name: str,
    instance_refresh_ids: Optional[List[str]] = None,
    next_token: Optional[str] = None,
    max_records: Optional[int] = None,
) -> Dict[str, Any]:
    params = {"AutoScalingGroupName": auto_scaling_group_name}
    if instance_refresh_ids:
        params["InstanceRefreshIds"] = instance_refresh_ids
    if next_token:
        params["NextToken"] = next_token
    if max_records:
        params["MaxRecords"] = max_records
    return client.describe_instance_refreshes(**params)


@AutoScalingErrorHandler.common_error_handler("start InstanceRefresh")
@AWSRetry.jittered_backoff(catch_extra_error_codes=["InstanceRefreshInProgress"])
def start_instance_refresh(client, auto_scaling_group_name: str, **params: Dict[str, Any]) -> str:
    return client.start_instance_refresh(AutoScalingGroupName=auto_scaling_group_name, **params).get(
        "InstanceRefreshId"
    )


@AutoScalingErrorHandler.common_error_handler("cancel InstanceRefresh")
@AWSRetry.jittered_backoff(catch_extra_error_codes=["InstanceRefreshInProgress"])
def cancel_instance_refresh(client, auto_scaling_group_name: str) -> str:
    return client.cancel_instance_refresh(AutoScalingGroupName=auto_scaling_group_name).get("InstanceRefreshId")



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: flexible_report_content_v1_info
short_description: Information module for Flexible Report Content V1
description:
- Get Flexible Report Content V1 by id.
- >
   This is used to download the flexible report. The API returns report content. Save the response to a file by
   converting the response data as a blob and setting the file format available from content-disposition response
   header.
version_added: '6.14.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  reportId:
    description:
    - ReportId path parameter. Id of the report.
    type: str
  executionId:
    description:
    - ExecutionId path parameter. Id of execution.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Reports DownloadFlexibleReportV1
  description: Complete reference of the DownloadFlexibleReportV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!download-flexible-report
notes:
  - SDK Method used are
    reports.Reports.download_flexible_report_v1,

  - Paths used are
    get /dna/data/api/v1/flexible-report/report/content/{reportId}/{executionId},

"""

EXAMPLES = r"""
- name: Get Flexible Report Content V1 by id
  cisco.dnac.flexible_report_content_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    reportId: string
    executionId: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: str
  sample: >
    "'string'"
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type
from ansible.plugins.action import ActionBase
try:
    from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
        AnsibleArgSpecValidator,
    )
except ImportError:
    ANSIBLE_UTILS_IS_INSTALLED = False
else:
    ANSIBLE_UTILS_IS_INSTALLED = True
from ansible.errors import AnsibleActionFail
from ansible_collections.cisco.dnac.plugins.plugin_utils.dnac import (
    DNACSDK,
    dnac_argument_spec,
)

# Get common arguments specification
argument_spec = dnac_argument_spec()
# Add arguments specific for this module
argument_spec.update(dict(
    eventId=dict(type="str"),
    tags=dict(type="str"),
    offset=dict(type="float"),
    limit=dict(type="float"),
    sortBy=dict(type="str"),
    order=dict(type="str"),
    headers=dict(type="dict"),
))

required_if = []
required_one_of = []
mutually_exclusive = []
required_together = []


class ActionModule(ActionBase):
    def __init__(self, *args, **kwargs):
        if not ANSIBLE_UTILS_IS_INSTALLED:
            raise AnsibleActionFail("ansible.utils is not installed. Execute 'ansible-galaxy collection install ansible.utils'")
        super(ActionModule, self).__init__(*args, **kwargs)
        self._supports_async = False
        self._supports_check_mode = True
        self._result = None

    # Checks the supplied parameters against the argument spec for this module
    def _check_argspec(self):
        aav = AnsibleArgSpecValidator(
            data=self._task.args,
            schema=dict(argument_spec=argument_spec),
            schema_format="argspec",
            schema_conditionals=dict(
                required_if=required_if,
                required_one_of=required_one_of,
                mutually_exclusive=mutually_exclusive,
                required_together=required_together,
            ),
            name=self._task.action,
        )
        valid, errors, self._task.args = aav.validate()
        if not valid:
            raise AnsibleActionFail(errors)

    def get_object(self, params):
        new_object = dict(
            event_id=params.get("eventId"),
            tags=params.get("tags"),
            offset=params.get("offset"),
            limit=params.get("limit"),
            sort_by=params.get("sortBy"),
            order=params.get("order"),
            headers=params.get("headers"),
        )
        return new_object

    def run(self, tmp=None, task_vars=None):
        self._task.diff = False
        self._result = super(ActionModule, self).run(tmp, task_vars)
        self._result["changed"] = False
        self._check_argspec()

        self._result.update(dict(dnac_response={}))

        dnac = DNACSDK(params=self._task.args)

        response = dnac.exec(
            family="event_management",
            function='get_events_v1',
            params=self.get_object(self._task.args),
        )
        self._result.update(dict(dnac_response=response))
        self._result.update(dnac.exit_json())
        return self._result



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: itsm_integration_events_failed_v1
short_description: Resource module for Itsm Integration Events Failed V1
description:
- Manage operation create of the resource Itsm Integration Events Failed V1.
- >
   Allows retry of multiple failed ITSM event instances. The retry request payload can be given as a list of strings
   "instance1","instance2","instance3",.. A minimum of one instance Id is mandatory. The list of failed event
   instance Ids can be retrieved using the 'Get Failed ITSM Events' API in the 'instanceId' attribute.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  payload:
    description: Itsm Integration Events Failed's payload.
    elements: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for ITSM RetryIntegrationEventsV1
  description: Complete reference of the RetryIntegrationEventsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!retry-integration-events
notes:
  - SDK Method used are
    itsm.Itsm.retry_integration_events_v1,

  - Paths used are
    post /dna/intent/api/v1/integration/events,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.itsm_integration_events_failed_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "executionId": "string",
      "executionStatusUrl": "string",
      "message": "string"
    }
"""



# (c) 2016 Red Hat Inc.
# (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
author:
 - Ansible Networking Team (@ansible-network)
name: network_cli
short_description: Use network_cli to run command on network appliances
description:
- This connection plugin provides a connection to remote devices over the SSH and
  implements a CLI shell.  This connection plugin is typically used by network devices
  for sending and receiving CLi commands to network devices.
version_added: 1.0.0
requirements:
- ansible-pylibssh if using I(ssh_type=libssh)
extends_documentation_fragment:
- ansible.netcommon.connection_persistent
options:
  host:
    description:
    - Specifies the remote device FQDN or IP address to establish the SSH connection
      to.
    default: inventory_hostname
    type: string
    vars:
    - name: inventory_hostname
    - name: ansible_host
  port:
    type: int
    description:
    - Specifies the port on the remote device that listens for connections when establishing
      the SSH connection.
    default: 22
    ini:
    - section: defaults
      key: remote_port
    env:
    - name: ANSIBLE_REMOTE_PORT
    vars:
    - name: ansible_port
  network_os:
    description:
    - Configures the device platform network operating system.  This value is used
      to load the correct terminal and cliconf plugins to communicate with the remote
      device.
    type: string
    vars:
    - name: ansible_network_os
  remote_user:
    description:
    - The username used to authenticate to the remote device when the SSH connection
      is first established.  If the remote_user is not specified, the connection will
      use the username of the logged in user.
    - Can be configured from the CLI via the C(--user) or C(-u) options.
    type: string
    ini:
    - section: defaults
      key: remote_user
    env:
    - name: ANSIBLE_REMOTE_USER
    vars:
    - name: ansible_user
  password:
    description:
    - Configures the user password used to authenticate to the remote device when
      first establishing the SSH connection.
    type: string
    vars:
    - name: ansible_password
    - name: ansible_ssh_pass
    - name: ansible_ssh_password
  private_key_file:
    description:
    - The private SSH key or certificate file used to authenticate to the remote device
      when first establishing the SSH connection.
    type: string
    ini:
    - section: defaults
      key: private_key_file
    env:
    - name: ANSIBLE_PRIVATE_KEY_FILE
    vars:
    - name: ansible_private_key_file
  become:
    type: boolean
    description:
    - The become option will instruct the CLI session to attempt privilege escalation
      on platforms that support it.  Normally this means transitioning from user mode
      to C(enable) mode in the CLI session. If become is set to True and the remote
      device does not support privilege escalation or the privilege has already been
      elevated, then this option is silently ignored.
    - Can be configured from the CLI via the C(--become) or C(-b) options.
    default: false
    ini:
    - section: privilege_escalation
      key: become
    env:
    - name: ANSIBLE_BECOME
    vars:
    - name: ansible_become
  become_errors:
    type: str
    description:
    - This option determines how privilege escalation failures are handled when
      I(become) is enabled.
    - When set to C(ignore), the errors are silently ignored.
      When set to C(warn), a warning message is displayed.
      The default option C(fail), triggers a failure and halts execution.
    vars:
    - name: ansible_network_become_errors
    default: fail
    choices: ["ignore", "warn", "fail"]
  terminal_errors:
    type: str
    description:
    - This option determines how failures while setting terminal parameters
      are handled.
    - When set to C(ignore), the errors are silently ignored.
      When set to C(warn), a warning message is displayed.
      The default option C(fail), triggers a failure and halts execution.
    vars:
    - name: ansible_network_terminal_errors
    default: fail
    choices: ["ignore", "warn", "fail"]
    version_added: 3.1.0
  become_method:
    description:
    - This option allows the become method to be specified in for handling privilege
      escalation.  Typically the become_method value is set to C(enable) but could
      be defined as other values.
    default: sudo
    type: string
    ini:
    - section: privilege_escalation
      key: become_method
    env:
    - name: ANSIBLE_BECOME_METHOD
    vars:
    - name: ansible_become_method
  host_key_auto_add:
    type: boolean
    description:
    - By default, Ansible will prompt the user before adding SSH keys to the known
      hosts file.  Since persistent connections such as network_cli run in background
      processes, the user will never be prompted.  By enabling this option, unknown
      host keys will automatically be added to the known hosts file.
    - Be sure to fully understand the security implications of enabling this option
      on production systems as it could create a security vulnerability.
    default: false
    ini:
    - section: paramiko_connection
      key: host_key_auto_add
    env:
    - name: ANSIBLE_HOST_KEY_AUTO_ADD
  persistent_buffer_read_timeout:
    type: float
    description:
    - Configures, in seconds, the amount of time to wait for the data to be read from
      Paramiko channel after the command prompt is matched. This timeout value ensures
      that command prompt matched is correct and there is no more data left to be
      received from remote host.
    default: 0.1
    ini:
    - section: persistent_connection
      key: buffer_read_timeout
    env:
    - name: ANSIBLE_PERSISTENT_BUFFER_READ_TIMEOUT
    vars:
    - name: ansible_buffer_read_timeout
  terminal_stdout_re:
    type: list
    elements: dict
    description:
    - A single regex pattern or a sequence of patterns along with optional flags to
      match the command prompt from the received response chunk. This option accepts
      C(pattern) and C(flags) keys. The value of C(pattern) is a python regex pattern
      to match the response and the value of C(flags) is the value accepted by I(flags)
      argument of I(re.compile) python method to control the way regex is matched
      with the response, for example I('re.I').
    vars:
    - name: ansible_terminal_stdout_re
  terminal_stderr_re:
    type: list
    elements: dict
    description:
    - This option provides the regex pattern and optional flags to match the error
      string from the received response chunk. This option accepts C(pattern) and
      C(flags) keys. The value of C(pattern) is a python regex pattern to match the
      response and the value of C(flags) is the value accepted by I(flags) argument
      of I(re.compile) python method to control the way regex is matched with the
      response, for example I('re.I').
    vars:
    - name: ansible_terminal_stderr_re
  terminal_initial_prompt:
    type: list
    elements: string
    description:
    - A single regex pattern or a sequence of patterns to evaluate the expected prompt
      at the time of initial login to the remote host.
    vars:
    - name: ansible_terminal_initial_prompt
  terminal_initial_answer:
    type: list
    elements: string
    description:
    - The answer to reply with if the C(terminal_initial_prompt) is matched. The value
      can be a single answer or a list of answers for multiple terminal_initial_prompt.
      In case the login menu has multiple prompts the sequence of the prompt and excepted
      answer should be in same order and the value of I(terminal_prompt_checkall)
      should be set to I(True) if all the values in C(terminal_initial_prompt) are
      expected to be matched and set to I(False) if any one login prompt is to be
      matched.
    vars:
    - name: ansible_terminal_initial_answer
  terminal_initial_prompt_checkall:
    type: boolean
    description:
    - By default the value is set to I(False) and any one of the prompts mentioned
      in C(terminal_initial_prompt) option is matched it won't check for other prompts.
      When set to I(True) it will check for all the prompts mentioned in C(terminal_initial_prompt)
      option in the given order and all the prompts should be received from remote
      host if not it will result in timeout.
    default: false
    vars:
    - name: ansible_terminal_initial_prompt_checkall
  terminal_inital_prompt_newline:
    type: boolean
    description:
    - This boolean flag, that when set to I(True) will send newline in the response
      if any of values in I(terminal_initial_prompt) is matched.
    default: true
    vars:
    - name: ansible_terminal_initial_prompt_newline
  network_cli_retries:
    description:
    - Number of attempts to connect to remote host. The delay time between the retires
      increases after every attempt by power of 2 in seconds till either the maximum
      attempts are exhausted or any of the C(persistent_command_timeout) or C(persistent_connect_timeout)
      timers are triggered.
    default: 3
    type: integer
    env:
    - name: ANSIBLE_NETWORK_CLI_RETRIES
    ini:
    - section: persistent_connection
      key: network_cli_retries
    vars:
    - name: ansible_network_cli_retries
  ssh_type:
    description:
      - The python package that will be used by the C(network_cli) connection plugin to create a SSH connection to remote host.
      - I(libssh) will use the ansible-pylibssh package, which needs to be installed in order to work.
      - I(paramiko) will instead use the paramiko package to manage the SSH connection.
      - I(auto) will use ansible-pylibssh if that package is installed, otherwise will fallback to paramiko.
    default: auto
    choices: ["libssh", "paramiko", "auto"]
    type: string
    env:
        - name: ANSIBLE_NETWORK_CLI_SSH_TYPE
    ini:
        - section: persistent_connection
          key: ssh_type
    vars:
    - name: ansible_network_cli_ssh_type
  host_key_checking:
    description: 'Set this to "False" if you want to avoid host key checking by the underlying tools Ansible uses to connect to the host'
    type: boolean
    default: True
    env:
    - name: ANSIBLE_HOST_KEY_CHECKING
    - name: ANSIBLE_SSH_HOST_KEY_CHECKING
    ini:
    - section: defaults
      key: host_key_checking
    - section: persistent_connection
      key: host_key_checking
    vars:
    - name: ansible_host_key_checking
    - name: ansible_ssh_host_key_checking
  single_user_mode:
    type: boolean
    default: false
    version_added: 2.0.0
    description:
    - This option enables caching of data fetched from the target for re-use.
      The cache is invalidated when the target device enters configuration mode.
    - Applicable only for platforms where this has been implemented.
    env:
    - name: ANSIBLE_NETWORK_SINGLE_USER_MODE
    vars:
    - name: ansible_network_single_user_mode
"""

import getpass
import json
import logging
import os
import re
import signal
import socket
import time
import traceback

from functools import wraps
from io import BytesIO

from ansible.errors import AnsibleConnectionFailure, AnsibleError
from ansible.module_utils._text import to_bytes, to_text
from ansible.module_utils.basic import missing_required_lib
from ansible.module_utils.six import PY3
from ansible.module_utils.six.moves import cPickle
from ansible.playbook.play_context import PlayContext
from ansible.plugins.loader import cache_loader, cliconf_loader, connection_loader, terminal_loader
from ansible.utils.display import Display

from ansible_collections.ansible.netcommon.plugins.connection.libssh import HAS_PYLIBSSH
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import to_list
from ansible_collections.ansible.netcommon.plugins.plugin_utils.connection_base import (
    NetworkConnectionBase,
)


try:
    from scp import SCPClient

    HAS_SCP = True
except ImportError:
    HAS_SCP = False
display = Display()


def ensure_connect(func):
    @wraps(func)
    def wrapped(self, *args, **kwargs):
        if not self._connected:
            self._connect()
        self.update_cli_prompt_context()
        return func(self, *args, **kwargs)

    return wrapped


class AnsibleCmdRespRecv(Exception):
    pass


class Connection(NetworkConnectionBase):
    """CLI (shell) SSH connections on Paramiko"""

    transport = "ansible.netcommon.network_cli"
    has_pipelining = True

    def __init__(self, play_context, new_stdin, *args, **kwargs):
        super(Connection, self).__init__(play_context, new_stdin, *args, **kwargs)
        self._ssh_shell = None

        self._matched_prompt = None
        self._matched_cmd_prompt = None
        self._matched_pattern = None
        self._last_response = None
        self._history = list()
        self._command_response = None
        self._last_recv_window = None
        self._cache = None

        self._terminal = None
        self.cliconf = None

        # Managing prompt context
        self._check_prompt = False

        self._task_uuid = to_text(kwargs.get("task_uuid", ""))
        self._ssh_type_conn = None
        self._ssh_type = None

        self._single_user_mode = False

        if self._network_os:
            self._terminal = terminal_loader.get(self._network_os, self)
            if not self._terminal:
                raise AnsibleConnectionFailure("network os %s is not supported" % self._network_os)

            self.cliconf = cliconf_loader.get(self._network_os, self)
            if not self.cliconf:
                self.queue_message(
                    "vvvv",
                    "unable to load cliconf for network_os %s. Falling back to default"
                    % self._network_os,
                )
                self.cliconf = cliconf_loader.get("ansible.netcommon.default", self)
                if not self.cliconf:
                    raise AnsibleConnectionFailure("Couldn't load fallback cliconf plugin")

            self._sub_plugin = {
                "type": "cliconf",
                "name": self.cliconf._load_name,
                "obj": self.cliconf,
            }
            self.queue_message(
                "vvvv",
                "loaded cliconf plugin %s from path %s for network_os %s"
                % (
                    self.cliconf._load_name,
                    self.cliconf._original_path,
                    self._network_os,
                ),
            )
        else:
            raise AnsibleConnectionFailure(
                "Unable to automatically determine host network os. Please "
                "manually configure ansible_network_os value for this host"
            )
        self.queue_message("log", "network_os is set to %s" % self._network_os)

    @property
    def ssh_type(self):
        if self._ssh_type is None:
            self._ssh_type = self.get_option("ssh_type")
            self.queue_message("vvvv", "ssh type is set to %s" % self._ssh_type)
            # Support autodetection of supported library
            if self._ssh_type == "auto":
                self.queue_message("vvvv", "autodetecting ssh_type")
                if HAS_PYLIBSSH:
                    self._ssh_type = "libssh"
                else:
                    self.queue_message(
                        "warning",
                        "ansible-pylibssh not installed, falling back to paramiko",
                    )
                    self._ssh_type = "paramiko"
                self.queue_message("vvvv", "ssh type is now set to %s" % self._ssh_type)

        if self._ssh_type not in ["paramiko", "libssh"]:
            raise AnsibleConnectionFailure(
                "Invalid value '%s' set for ssh_type option."
                " Expected value is either 'libssh' or 'paramiko'" % self._ssh_type
            )

        return self._ssh_type

    @property
    def ssh_type_conn(self):
        if self._ssh_type_conn is None:
            if self.ssh_type == "libssh":
                connection_plugin = "ansible.netcommon.libssh"
            elif self.ssh_type == "paramiko":
                # NOTE: This MUST be paramiko or things will break
                connection_plugin = "paramiko"
            else:
                raise AnsibleConnectionFailure(
                    "Invalid value '%s' set for ssh_type option."
                    " Expected value is either 'libssh' or 'paramiko'" % self._ssh_type
                )

            self._ssh_type_conn = connection_loader.get(
                connection_plugin, self._play_context, "/dev/null"
            )

        return self._ssh_type_conn

    # To maintain backward compatibility
    @property
    def paramiko_conn(self):
        return self.ssh_type_conn

    def _get_log_channel(self):
        user = ""
        try:
            user = getpass.getuser()
        except KeyError:
            self.queue_message(
                "vv",
                "Current user (uid=%s) does not seem to exist on this system, leaving user empty."
                % os.getuid(),
            )
        name = "p=%s u=%s | " % (os.getpid(), user)
        name += "%s [%s]" % (self.ssh_type, self._play_context.remote_addr)
        return name

    @ensure_connect
    def get_prompt(self):
        """Returns the current prompt from the device"""
        return self._matched_prompt

    def exec_command(self, cmd, in_data=None, sudoable=True):
        # not accessible code alert can be taken out at around  01-01-2027,
        # when connection local is removed
        if self._ssh_shell:
            try:
                cmd = json.loads(to_text(cmd, errors="surrogate_or_strict"))
                kwargs = {"command": to_bytes(cmd["command"], errors="surrogate_or_strict")}
                for key in (
                    "prompt",
                    "answer",
                    "sendonly",
                    "newline",
                    "prompt_retry_check",
                ):
                    if cmd.get(key) is True or cmd.get(key) is False:
                        kwargs[key] = cmd[key]
                    elif cmd.get(key) is not None:
                        kwargs[key] = to_bytes(cmd[key], errors="surrogate_or_strict")
                return self.send(**kwargs)
            except ValueError:
                cmd = to_bytes(cmd, errors="surrogate_or_strict")
                return self.send(command=cmd)

        else:
            return super(Connection, self).exec_command(cmd, in_data, sudoable)

    def get_options(self, hostvars=None):
        options = super(Connection, self).get_options(hostvars=hostvars)
        options.update(self.ssh_type_conn.get_options(hostvars=hostvars))
        return options

    def set_options(self, task_keys=None, var_options=None, direct=None):
        super(Connection, self).set_options(
            task_keys=task_keys, var_options=var_options, direct=direct
        )
        self.ssh_type_conn.set_options(task_keys=task_keys, var_options=var_options, direct=direct)
        # Retain old look_for_keys behaviour, but only if not set
        if not any(
            [
                task_keys and ("look_for_keys" in task_keys),
                var_options and ("look_for_keys" in var_options),
                direct and ("look_for_keys" in direct),
            ]
        ):
            look_for_keys = not bool(
                self.get_option("password") and not self.get_option("private_key_file")
            )
            if not look_for_keys:
                # This actually can't be overridden yet without changes in ansible-core
                # TODO: Uncomment when appropriate
                # self.queue_message(
                #     "warning",
                #     "Option look_for_keys has been implicitly set to {0} because "
                #     "it was not set explicitly. This is retained to maintain "
                #     "backwards compatibility with the old behavior. This behavior "
                #     "will be removed in some release after 2024-01-01".format(
                #         look_for_keys
                #     ),
                # )
                self.ssh_type_conn.set_option("look_for_keys", look_for_keys)

    def update_play_context(self, pc_data):
        """Updates the play context information for the connection"""
        pc_data = to_bytes(pc_data)
        if PY3:
            pc_data = cPickle.loads(pc_data, encoding="bytes")
        else:
            pc_data = cPickle.loads(pc_data)
        play_context = PlayContext()
        play_context.deserialize(pc_data)

        self.queue_message("vvvv", "updating play_context for connection")
        if self._play_context.become ^ play_context.become:
            if play_context.become is True:
                auth_pass = play_context.become_pass
                self._on_become(become_pass=auth_pass)
                self.queue_message("vvvv", "authorizing connection")
            else:
                self._terminal.on_unbecome()
                self.queue_message("vvvv", "deauthorizing connection")

        self._play_context = play_context
        if self._ssh_type_conn is not None:
            # TODO: This works, but is not really ideal. We would rather use
            #       set_options, but then we need more custom handling in that
            #       method.
            self._ssh_type_conn._play_context = play_context

        if hasattr(self, "reset_history"):
            self.reset_history()
        if hasattr(self, "disable_response_logging"):
            self.disable_response_logging()

        self._single_user_mode = self.get_option("single_user_mode")

    def set_check_prompt(self, task_uuid):
        self._check_prompt = task_uuid

    def update_cli_prompt_context(self):
        # set cli prompt context at the start of new task run only
        if self._check_prompt and self._task_uuid != self._check_prompt:
            self._task_uuid, self._check_prompt = self._check_prompt, False
            self.set_cli_prompt_context()

    def _connect(self):
        """
        Connects to the remote device and starts the terminal
        """
        if display.verbosity > 3:
            logging.getLogger(self.ssh_type).setLevel(logging.DEBUG)

        self.queue_message("vvvv", "invoked shell using ssh_type: %s" % self.ssh_type)

        self._single_user_mode = self.get_option("single_user_mode")

        if not self.connected:
            self.ssh_type_conn._set_log_channel(self._get_log_channel())
            self.ssh_type_conn.force_persistence = self.force_persistence

            command_timeout = self.get_option("persistent_command_timeout")
            max_pause = min(
                [
                    self.get_option("persistent_connect_timeout"),
                    command_timeout,
                ]
            )
            retries = self.get_option("network_cli_retries")
            total_pause = 0

            for attempt in range(retries + 1):
                try:
                    ssh = self.ssh_type_conn._connect()
                    break
                except AnsibleError:
                    raise
                except Exception as e:
                    pause = 2 ** (attempt + 1)
                    if attempt == retries or total_pause >= max_pause:
                        raise AnsibleConnectionFailure(to_text(e, errors="surrogate_or_strict"))
                    else:
                        msg = (
                            "network_cli_retry: attempt: %d, caught exception(%s), "
                            "pausing for %d seconds"
                            % (
                                attempt + 1,
                                to_text(e, errors="surrogate_or_strict"),
                                pause,
                            )
                        )

                        self.queue_message("vv", msg)
                        time.sleep(pause)
                        total_pause += pause
                        continue

            self.queue_message("vvvv", "ssh connection done, setting terminal")
            self._connected = True

            self._ssh_shell = ssh.ssh.invoke_shell()
            if self.ssh_type == "paramiko":
                self._ssh_shell.settimeout(command_timeout)

            self.queue_message(
                "vvvv",
                "loaded terminal plugin for network_os %s" % self._network_os,
            )

            terminal_initial_prompt = (
                self.get_option("terminal_initial_prompt") or self._terminal.terminal_initial_prompt
            )
            terminal_initial_answer = (
                self.get_option("terminal_initial_answer") or self._terminal.terminal_initial_answer
            )
            newline = (
                self.get_option("terminal_inital_prompt_newline")
                or self._terminal.terminal_inital_prompt_newline
            )
            check_all = self.get_option("terminal_initial_prompt_checkall") or False

            self.receive(
                prompts=terminal_initial_prompt,
                answer=terminal_initial_answer,
                newline=newline,
                check_all=check_all,
            )

            if self._play_context.become:
                self.queue_message("vvvv", "firing event: on_become")
                auth_pass = self._play_context.become_pass
                self._on_become(become_pass=auth_pass)

            self.queue_message("vvvv", "firing event: on_open_shell()")
            self._on_open_shell()

            self.queue_message("vvvv", "ssh connection has completed successfully")

        return self

    def _on_become(self, become_pass=None):
        """
        Wraps terminal.on_become() to handle
        privilege escalation failures based on user preference
        """
        on_become_error = self.get_option("become_errors")
        try:
            self._terminal.on_become(passwd=become_pass)
        except AnsibleConnectionFailure:
            if on_become_error == "ignore":
                pass
            elif on_become_error == "warn":
                self.queue_message("warning", "on_become: privilege escalation failed")
            else:
                raise

    def _on_open_shell(self):
        """
        Wraps terminal.on_open_shell() to handle
        terminal setting failures based on user preference
        """
        on_terminal_error = self.get_option("terminal_errors")
        try:
            self._terminal.on_open_shell()
        except AnsibleConnectionFailure:
            if on_terminal_error == "ignore":
                pass
            elif on_terminal_error == "warn":
                self.queue_message(
                    "warning",
                    "on_open_shell: failed to set terminal parameters",
                )
            else:
                raise

    def close(self):
        """
        Close the active connection to the device
        """
        # only close the connection if its connected.
        if self._connected:
            self.queue_message("debug", "closing ssh connection to device")
            if self._ssh_shell:
                self.queue_message("debug", "firing event: on_close_shell()")
                self._terminal.on_close_shell()
                self._ssh_shell.close()
                self._ssh_shell = None
                self.queue_message("debug", "cli session is now closed")

                self.ssh_type_conn.close()
                self._ssh_type_conn = None
                self.queue_message("debug", "ssh connection has been closed successfully")
        super(Connection, self).close()

    def _read_post_command_prompt_match(self):
        time.sleep(self.get_option("persistent_buffer_read_timeout"))
        data = self._ssh_shell.read_bulk_response()
        return data if data else None

    def receive_paramiko(
        self,
        command=None,
        prompts=None,
        answer=None,
        newline=True,
        prompt_retry_check=False,
        check_all=False,
        strip_prompt=True,
    ):
        recv = BytesIO()
        cache_socket_timeout = self.get_option("persistent_command_timeout")
        self._ssh_shell.settimeout(cache_socket_timeout)
        command_prompt_matched = False
        handled = False
        errored_response = None

        while True:
            if command_prompt_matched:
                try:
                    signal.signal(signal.SIGALRM, self._handle_buffer_read_timeout)
                    signal.setitimer(signal.ITIMER_REAL, self._buffer_read_timeout)
                    data = self._ssh_shell.recv(256)
                    signal.alarm(0)
                    self._log_messages("response-%s: %s" % (self._window_count + 1, data))
                    # if data is still received on channel it indicates the prompt string
                    # is wrongly matched in between response chunks, continue to read
                    # remaining response.
                    command_prompt_matched = False

                    # restart command_timeout timer
                    signal.signal(signal.SIGALRM, self._handle_command_timeout)
                    signal.alarm(self._command_timeout)

                except AnsibleCmdRespRecv:
                    # reset socket timeout to global timeout
                    return self._command_response
            else:
                data = self._ssh_shell.recv(256)
                self._log_messages("response-%s: %s" % (self._window_count + 1, data))
            # when a channel stream is closed, received data will be empty
            if not data:
                break

            recv.write(data)
            offset = recv.tell() - 256 if recv.tell() > 256 else 0
            recv.seek(offset)

            window = self._strip(recv.read())
            self._last_recv_window = window
            self._window_count += 1

            if prompts and not handled:
                handled = self._handle_prompt(window, prompts, answer, newline, False, check_all)
                self._matched_prompt_window = self._window_count
            elif (
                prompts
                and handled
                and prompt_retry_check
                and self._matched_prompt_window + 1 == self._window_count
            ):
                # check again even when handled, if same prompt repeats in next window
                # (like in the case of a wrong enable password, etc) indicates
                # value of answer is wrong, report this as error.
                if self._handle_prompt(
                    window,
                    prompts,
                    answer,
                    newline,
                    prompt_retry_check,
                    check_all,
                ):
                    raise AnsibleConnectionFailure(
                        "For matched prompt '%s', answer is not valid" % self._matched_cmd_prompt
                    )

            if self._find_error(window):
                # We can't exit here, as we need to drain the buffer in case
                # the error isn't fatal, and will be using the buffer again
                errored_response = window

            if self._find_prompt(window):
                if errored_response:
                    raise AnsibleConnectionFailure(errored_response)
                self._last_response = recv.getvalue()
                resp = self._strip(self._last_response)
                self._command_response = self._sanitize(resp, command, strip_prompt)
                if self._buffer_read_timeout == 0.0:
                    # reset socket timeout to global timeout
                    return self._command_response
                else:
                    command_prompt_matched = True

    def receive_libssh(
        self,
        command=None,
        prompts=None,
        answer=None,
        newline=True,
        prompt_retry_check=False,
        check_all=False,
        strip_prompt=True,
    ):
        self._command_response = resp = b""
        command_prompt_matched = False
        handled = False
        errored_response = None

        while True:
            if command_prompt_matched:
                data = self._read_post_command_prompt_match()
                if data:
                    command_prompt_matched = False
                else:
                    return self._command_response
            else:
                try:
                    data = self._ssh_shell.read_bulk_response()
                # TODO: Should be ConnectionError when pylibssh drops Python 2 support
                except OSError:
                    # Socket has closed
                    break

            if not data:
                continue
            self._last_recv_window = self._strip(data)
            resp += self._last_recv_window
            self._window_count += 1

            self._log_messages("response-%s: %s" % (self._window_count, data))

            if prompts and not handled:
                handled = self._handle_prompt(resp, prompts, answer, newline, False, check_all)
                self._matched_prompt_window = self._window_count
            elif (
                prompts
                and handled
                and prompt_retry_check
                and self._matched_prompt_window + 1 == self._window_count
            ):
                # check again even when handled, if same prompt repeats in next window
                # (like in the case of a wrong enable password, etc) indicates
                # value of answer is wrong, report this as error.
                if self._handle_prompt(
                    resp,
                    prompts,
                    answer,
                    newline,
                    prompt_retry_check,
                    check_all,
                ):
                    raise AnsibleConnectionFailure(
                        "For matched prompt '%s', answer is not valid" % self._matched_cmd_prompt
                    )

            if self._find_error(resp):
                # We can't exit here, as we need to drain the buffer in case
                # the error isn't fatal, and will be using the buffer again
                errored_response = resp

            if self._find_prompt(resp):
                if errored_response:
                    raise AnsibleConnectionFailure(errored_response)
                self._last_response = data
                self._command_response = self._sanitize(resp, command, strip_prompt)
                command_prompt_matched = True

    def receive(
        self,
        command=None,
        prompts=None,
        answer=None,
        newline=True,
        prompt_retry_check=False,
        check_all=False,
        strip_prompt=True,
    ):
        """
        Handles receiving of output from command
        """
        self._matched_prompt = None
        self._matched_cmd_prompt = None
        self._matched_prompt_window = 0
        self._window_count = 0

        # set terminal regex values for command prompt and errors in response
        self._terminal_stderr_re = self._get_terminal_std_re("terminal_stderr_re")
        self._terminal_stdout_re = self._get_terminal_std_re("terminal_stdout_re")

        self._command_timeout = self.get_option("persistent_command_timeout")
        self._validate_timeout_value(self._command_timeout, "persistent_command_timeout")

        self._buffer_read_timeout = self.get_option("persistent_buffer_read_timeout")
        self._validate_timeout_value(self._buffer_read_timeout, "persistent_buffer_read_timeout")

        self._log_messages("command: %s" % command)
        if self.ssh_type == "libssh":
            response = self.receive_libssh(
                command,
                prompts,
                answer,
                newline,
                prompt_retry_check,
                check_all,
                strip_prompt,
            )
        elif self.ssh_type == "paramiko":
            response = self.receive_paramiko(
                command,
                prompts,
                answer,
                newline,
                prompt_retry_check,
                check_all,
                strip_prompt,
            )

        return response

    @ensure_connect
    def send(
        self,
        command,
        prompt=None,
        answer=None,
        newline=True,
        sendonly=False,
        prompt_retry_check=False,
        check_all=False,
        strip_prompt=True,
    ):
        """
        Sends the command to the device in the opened shell
        """
        # try cache first
        if (not prompt) and (self._single_user_mode):
            out = self.get_cache().lookup(command)
            if out:
                self.queue_message("vvvv", "cache hit for command: %s" % command)
                return out

        if check_all:
            prompt_len = len(to_list(prompt))
            answer_len = len(to_list(answer))
            if prompt_len != answer_len:
                raise AnsibleConnectionFailure(
                    "Number of prompts (%s) is not same as that of answers (%s)"
                    % (prompt_len, answer_len)
                )
        try:
            cmd = b"%s\r" % command
            self._history.append(cmd)
            self._ssh_shell.sendall(cmd)
            self._log_messages("send command: %s" % cmd)
            if sendonly:
                return
            response = self.receive(
                command,
                prompt,
                answer,
                newline,
                prompt_retry_check,
                check_all,
                strip_prompt,
            )
            response = to_text(response, errors="surrogate_then_replace")

            if (not prompt) and (self._single_user_mode):
                if self._needs_cache_invalidation(command):
                    # invalidate the existing cache
                    if self.get_cache().keys():
                        self.queue_message("vvvv", "invalidating existing cache")
                        self.get_cache().invalidate()
                else:
                    # populate cache
                    self.queue_message("vvvv", "populating cache for command: %s" % command)
                    self.get_cache().populate(command, response)

            return response
        except (socket.timeout, AttributeError):
            self.queue_message("error", traceback.format_exc())
            raise AnsibleConnectionFailure(
                "timeout value %s seconds reached while trying to send command: %s"
                % (self._ssh_shell.gettimeout(), command.strip())
            )

    def _handle_buffer_read_timeout(self, signum, frame):
        self.queue_message(
            "vvvv",
            "Response received, triggered 'persistent_buffer_read_timeout' timer of %s seconds"
            % self.get_option("persistent_buffer_read_timeout"),
        )
        raise AnsibleCmdRespRecv()

    def _handle_command_timeout(self, signum, frame):
        msg = (
            "command timeout triggered, timeout value is %s secs.\nSee the timeout setting options in the Network Debug and Troubleshooting Guide."
            % self.get_option("persistent_command_timeout")
        )
        self.queue_message("log", msg)
        raise AnsibleConnectionFailure(msg)

    def _strip(self, data):
        """
        Removes ANSI codes from device response
        """
        for regex in self._terminal.ansi_re:
            data = regex.sub(b"", data)
        return data

    def _handle_prompt(
        self,
        resp,
        prompts,
        answer,
        newline,
        prompt_retry_check=False,
        check_all=False,
    ):
        """
        Matches the command prompt and responds

        :arg resp: Byte string containing the raw response from the remote
        :arg prompts: Sequence of byte strings that we consider prompts for input
        :arg answer: Sequence of Byte string to send back to the remote if we find a prompt.
                A carriage return is automatically appended to this string.
        :param prompt_retry_check: Bool value for trying to detect more prompts
        :param check_all: Bool value to indicate if all the values in prompt sequence should be matched or any one of
                          given prompt.
        :returns: True if a prompt was found in ``resp``. If check_all is True
                  will True only after all the prompt in the prompts list are matched. False otherwise.
        """
        single_prompt = False
        if not isinstance(prompts, list):
            prompts = [prompts]
            single_prompt = True
        if not isinstance(answer, list):
            answer = [answer]
        try:
            prompts_regex = [re.compile(to_bytes(r), re.I) for r in prompts]
        except re.error as exc:
            raise ConnectionError(
                "Failed to compile one or more terminal prompt regexes: %s.\n"
                "Prompts provided: %s" % (to_text(exc), prompts)
            )
        for index, regex in enumerate(prompts_regex):
            match = regex.search(resp)
            if match:
                self._matched_cmd_prompt = match.group()
                self._log_messages("matched command prompt: %s" % self._matched_cmd_prompt)

                # if prompt_retry_check is enabled to check if same prompt is
                # repeated don't send answer again.
                if not prompt_retry_check:
                    prompt_answer = to_bytes(answer[index] if len(answer) > index else answer[0])
                    if newline:
                        prompt_answer += b"\r"
                    self._ssh_shell.sendall(prompt_answer)
                    self._log_messages("matched command prompt answer: %s" % prompt_answer)
                if check_all and prompts and not single_prompt:
                    prompts.pop(0)
                    answer.pop(0)
                    return False
                return True
        return False

    def _sanitize(self, resp, command=None, strip_prompt=True):
        """
        Removes elements from the response before returning to the caller
        """
        cleaned = []
        for line in resp.splitlines():
            if command and line.strip() == command.strip():
                continue

            for prompt in self._matched_prompt.strip().splitlines():
                if prompt.strip() in line and strip_prompt:
                    break
            else:
                cleaned.append(line)

        return b"\n".join(cleaned).strip()

    def _find_error(self, response):
        """Searches the buffered response for a matching error condition"""
        for stderr_regex in self._terminal_stderr_re:
            if stderr_regex.search(response):
                self._log_messages(
                    "matched error regex (terminal_stderr_re) '%s' from response '%s'"
                    % (stderr_regex.pattern, response)
                )

                self._log_messages(
                    "matched stdout regex (terminal_stdout_re) '%s' from error response '%s'"
                    % (self._matched_pattern, response)
                )
                return True

        return False

    def _find_prompt(self, response):
        """Searches the buffered response for a matching command prompt"""
        for stdout_regex in self._terminal_stdout_re:
            match = stdout_regex.search(response)
            if match:
                self._matched_pattern = stdout_regex.pattern
                self._matched_prompt = match.group()
                self._log_messages(
                    "matched cli prompt '%s' with regex '%s' from response '%s'"
                    % (self._matched_prompt, self._matched_pattern, response)
                )
                return True

        return False

    def _validate_timeout_value(self, timeout, timer_name):
        if timeout < 0:
            raise AnsibleConnectionFailure(
                "'%s' timer value '%s' is invalid, value should be greater than or equal to zero."
                % (timer_name, timeout)
            )

    def transport_test(self, connect_timeout):
        """This method enables wait_for_connection to work.

        As it is used by wait_for_connection, it is called by that module's action plugin,
        which is on the controller process, which means that nothing done on this instance
        should impact the actual persistent connection... this check is for informational
        purposes only and should be properly cleaned up.
        """

        # Force a fresh connect if for some reason we have connected before.
        self.close()
        self._connect()
        self.close()

    def _get_terminal_std_re(self, option):
        terminal_std_option = self.get_option(option)
        terminal_std_re = []

        if terminal_std_option:
            for item in terminal_std_option:
                if "pattern" not in item:
                    raise AnsibleConnectionFailure(
                        "'pattern' is a required key for option '%s',"
                        " received option value is %s" % (option, item)
                    )
                pattern = rb"%s" % to_bytes(item["pattern"])
                flag = item.get("flags", 0)
                if flag:
                    flag = getattr(re, flag.split(".")[1])
                terminal_std_re.append(re.compile(pattern, flag))
        else:
            # To maintain backward compatibility
            terminal_std_re = getattr(self._terminal, option)

        return terminal_std_re

    def copy_file(self, source=None, destination=None, proto="scp", timeout=30):
        """Copies file over scp/sftp to remote device

        :param source: Source file path
        :param destination: Destination file path on remote device
        :param proto: Protocol to be used for file transfer,
                      supported protocol: scp and sftp
        :param timeout: Specifies the wait time to receive response from
                        remote host before triggering timeout exception
        :return: None
        """
        ssh = self.ssh_type_conn._connect_uncached()
        if self.ssh_type == "libssh":
            self.ssh_type_conn.put_file(source, destination, proto=proto)
        elif self.ssh_type == "paramiko":
            if proto == "scp":
                if not HAS_SCP:
                    raise AnsibleError(missing_required_lib("scp"))
                with SCPClient(ssh.get_transport(), socket_timeout=timeout) as scp:
                    scp.put(source, destination)
            elif proto == "sftp":
                with ssh.open_sftp() as sftp:
                    sftp.put(source, destination)
            else:
                raise AnsibleError("Do not know how to do transfer file over protocol %s" % proto)
        else:
            raise AnsibleError("Do not know how to do SCP with ssh_type %s" % self.ssh_type)

    def get_file(self, source=None, destination=None, proto="scp", timeout=30):
        """Fetch file over scp/sftp from remote device
        :param source: Source file path
        :param destination: Destination file path
        :param proto: Protocol to be used for file transfer,
                      supported protocol: scp and sftp
        :param timeout: Specifies the wait time to receive response from
                        remote host before triggering timeout exception
        :return: None
        """
        """Fetch file over scp/sftp from remote device"""
        ssh = self.ssh_type_conn._connect_uncached()
        if self.ssh_type == "libssh":
            self.ssh_type_conn.fetch_file(source, destination, proto=proto)
        elif self.ssh_type == "paramiko":
            if proto == "scp":
                if not HAS_SCP:
                    raise AnsibleError(missing_required_lib("scp"))
                try:
                    with SCPClient(ssh.get_transport(), socket_timeout=timeout) as scp:
                        scp.get(source, destination)
                except EOFError:
                    # This appears to be benign.
                    pass
            elif proto == "sftp":
                with ssh.open_sftp() as sftp:
                    sftp.get(source, destination)
            else:
                raise AnsibleError("Do not know how to do transfer file over protocol %s" % proto)
        else:
            raise AnsibleError("Do not know how to do SCP with ssh_type %s" % self.ssh_type)

    def get_cache(self):
        if not self._cache:
            # TO-DO: support jsonfile or other modes of caching with
            #        a configurable option
            self._cache = cache_loader.get("ansible.netcommon.memory")
        return self._cache

    def _is_in_config_mode(self):
        """
        Check if the target device is in config mode by comparing
        the current prompt with the platform's `terminal_config_prompt`.
        Returns False if `terminal_config_prompt` is not defined.

        :returns: A boolean indicating if the device is in config mode or not.
        """
        cfg_mode = False
        cur_prompt = to_text(self.get_prompt(), errors="surrogate_then_replace").strip()
        cfg_prompt = getattr(self._terminal, "terminal_config_prompt", None)
        if cfg_prompt and cfg_prompt.match(cur_prompt):
            cfg_mode = True
        return cfg_mode

    def _needs_cache_invalidation(self, command):
        """
        This method determines if it is necessary to invalidate
        the existing cache based on whether the device has entered
        configuration mode or if the last command sent to the device
        is potentially capable of making configuration changes.

        :param command: The last command sent to the target device.
        :returns: A boolean indicating if cache invalidation is required or not.
        """
        invalidate = False
        cfg_cmds = []
        try:
            # AnsiblePlugin base class in Ansible 2.9 does not have has_option() method.
            # TO-DO: use has_option() when we drop 2.9 support.
            cfg_cmds = self.cliconf.get_option("config_commands")
        except AttributeError:
            cfg_cmds = []
        if (self._is_in_config_mode()) or (to_text(command) in cfg_cmds):
            invalidate = True
        return invalidate



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: sensor_test_template_edit
short_description: Resource module for Sensor Test Template Edit
description:
- This module represents an alias of the module sensor_test_template_edit_v1
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  _id:
    description: The sensor test template unique identifier, generated at test creation
      time.
    type: str
  actionInProgress:
    description: Indication of inprogress action.
    type: str
  apCoverage:
    description: Sensor Test Template Edit's apCoverage.
    elements: dict
    suboptions:
      bands:
        description: The WIFI bands.
        type: str
      numberOfApsToTest:
        description: Number of APs to test.
        type: int
      rssiThreshold:
        description: RSSI threshold.
        type: int
    type: list
  connection:
    description: Connection type of test WIRED, WIRELESS, BOTH.
    type: str
  encryptionMode:
    description: Encryption mode.
    type: str
  frequency:
    description: Sensor Test Template Edit's frequency.
    suboptions:
      unit:
        description: Unit of the time value (NANOSECONDS, MICROSECONDS, MILLISECONDS,
          SECONDS, MINUTES, HOURS, DAYS).
        type: str
      value:
        description: Value of the unit.
        type: int
    type: dict
  lastModifiedTime:
    description: Last modify time.
    type: int
  location:
    description: Location string.
    type: str
  locationInfoList:
    description: Sensor Test Template Edit's locationInfoList.
    elements: dict
    suboptions:
      allSensors:
        description: Use all sensors in the site for test.
        type: bool
      customManagementVlan:
        description: Custom Management VLAN.
        type: bool
      locationId:
        description: Site UUID.
        type: str
      locationType:
        description: Site type.
        type: str
      macAddressList:
        description: MAC addresses.
        elements: str
        type: list
      managementVlan:
        description: Management VLAN.
        type: str
      siteHierarchy:
        description: Site name hierarhy.
        type: str
    type: list
  modelVersion:
    description: Test template object model version (must be 2).
    type: int
  name:
    description: The sensor test template name, which is the same as in 'templateName'.
    type: str
  numAssociatedSensor:
    description: Number of associated sensor.
    type: int
  numNeighborAPThreshold:
    description: Number of neighboring AP threshold.
    type: int
  profiles:
    description: Sensor Test Template Edit's profiles.
    elements: dict
    suboptions:
      authProtocol:
        description: Auth protocol.
        type: str
      authType:
        description: Authentication type OPEN, WPA2_PSK, WPA2_EAP, WEB_AUTH, MAB, DOT1X,
          OTHER.
        type: str
      certdownloadurl:
        description: Certificate download URL.
        type: str
      certfilename:
        description: Auth certificate file name.
        type: str
      certpassphrase:
        description: Certificate password phrase.
        type: str
      certstatus:
        description: Certificate status INACTIVE or ACTIVE.
        type: str
      certxferprotocol:
        description: Certificate transfering protocol HTTP or HTTPS.
        type: str
      deviceType:
        description: Device Type.
        type: str
      eapMethod:
        description: WPA2_EAP methods EAP-FAST, PEAP-MSCHAPv2, EAP-TLS, PEAP-TLS, EAP-TTLS-MSCHAPv2,
          EAP-TTLS-PAP, EAP-TTLS-CHAP, EAP-FAST-GTC, EAP-PEAP-GTC.
        type: str
      extWebAuth:
        description: Indication of using external WEB Auth.
        type: bool
      extWebAuthAccessUrl:
        description: External WEB Auth access URL.
        type: str
      extWebAuthHtmlTag:
        description: Sensor Test Template Edit's extWebAuthHtmlTag.
        elements: dict
        suboptions:
          label:
            description: Label.
            type: str
          tag:
            description: Tag.
            type: str
          value:
            description: Value.
            type: str
        type: list
      extWebAuthPortal:
        description: External authentication portal.
        type: str
      extWebAuthVirtualIp:
        description: External WEB Auth virtual IP.
        type: str
      locationVlanList:
        description: Sensor Test Template Edit's locationVlanList.
        elements: dict
        suboptions:
          locationId:
            description: Site UUID.
            type: str
          vlans:
            description: Array of VLANs.
            elements: str
            type: list
        type: list
      password:
        description: Password string for onboarding SSID.
        type: str
      passwordType:
        description: SSID password type ASCII or HEX.
        type: str
      profileName:
        description: Profile name.
        type: str
      psk:
        description: Password of SSID when passwordType is ASCII.
        type: str
      qosPolicy:
        description: QoS policy PlATINUM, GOLD, SILVER, BRONZE.
        type: str
      scep:
        description: Secure certificate enrollment protocol true or false or null for
          not applicable.
        type: bool
      tests:
        description: Sensor Test Template Edit's tests.
        elements: dict
        suboptions:
          config:
            description: Sensor Test Template Edit's config.
            elements: dict
            suboptions:
              direction:
                description: IPerf direction (UPLOAD, DOWNLOAD, BOTH).
                type: str
              domains:
                description: DNS domain name.
                elements: str
                type: list
              downlinkTest:
                description: Downlink test.
                type: bool
              endPort:
                description: IPerf end port.
                type: int
              exitCommand:
                description: Exit command.
                type: str
              finalPrompt:
                description: Final prompt.
                type: str
              ndtServer:
                description: NDT server.
                type: str
              ndtServerPath:
                description: NDT server path.
                type: str
              ndtServerPort:
                description: NDT server port.
                type: str
              numPackets:
                description: Number of packets.
                type: int
              password:
                description: Password.
                type: str
              passwordPrompt:
                description: Password prompt.
                type: str
              pathToDownload:
                description: File path for file transfer.
                type: str
              port:
                description: Radius or WEB server port.
                type: int
              probeType:
                description: Probe type.
                type: str
              protocol:
                description: Protocol used by file transfer, IPerf, mail server, and
                  radius (TCP, UDP, FTP, POP3, IMAP, CHAP, PAP).
                type: str
              proxyPassword:
                description: Proxy password.
                type: str
              proxyPort:
                description: Proxy port.
                type: str
              proxyServer:
                description: Proxy server.
                type: str
              proxyUserName:
                description: Proxy user name.
                type: str
              server:
                description: Ping, file transfer, mail, radius, ssh, or telnet server.
                type: str
              servers:
                description: IPerf server list.
                elements: str
                type: list
              sharedSecret:
                description: Shared secret.
                type: str
              startPort:
                description: IPerf start port.
                type: int
              transferType:
                description: File transfer type (UPLOAD, DOWNLOAD, BOTH).
                type: str
              udpBandwidth:
                description: IPerf UDP bandwidth.
                type: int
              uplinkTest:
                description: Uplink test.
                type: bool
              url:
                description: URL.
                type: str
              userName:
                description: User name.
                type: str
              userNamePrompt:
                description: User name prompt.
                type: str
            type: list
          name:
            description: Name of the test.
            type: str
        type: list
      username:
        description: User name string for onboarding SSID.
        type: str
      vlan:
        description: VLAN.
        type: str
      whiteList:
        description: Indication of being on allowed list.
        type: bool
    type: list
  radioAsSensorRemoved:
    description: Radio as sensor removed.
    type: bool
  rssiThreshold:
    description: RSSI threshold.
    type: int
  runNow:
    description: Run now (YES, NO).
    type: str
  scheduleInDays:
    description: Bit-wise value of scheduled test days.
    type: int
  sensors:
    description: Sensor Test Template Edit's sensors.
    elements: dict
    suboptions:
      allSensorAddition:
        description: Is all sensor addition.
        type: bool
      assigned:
        description: Is assigned.
        type: bool
      configUpdated:
        description: Configuration updated YES, NO.
        type: str
      hostName:
        description: Host name.
        type: str
      iPerfInfo:
        description: A string-stringList iPerf information.
        type: dict
      id:
        description: Sensor ID.
        type: str
      ipAddress:
        description: IP address.
        type: str
      locationId:
        description: Site UUID.
        type: str
      macAddress:
        description: MAC address.
        type: str
      markedForUninstall:
        description: Is marked for uninstall.
        type: bool
      name:
        description: Sensor name.
        type: str
      runNow:
        description: Run now YES, NO.
        type: str
      sensorType:
        description: Sensor type.
        type: str
      servicePolicy:
        description: Service policy.
        type: str
      status:
        description: Sensor device status UP, DOWN, REBOOT.
        type: str
      switchMac:
        description: Switch MAC address.
        type: str
      switchSerialNumber:
        description: Switch serial number.
        type: str
      switchUuid:
        description: Switch device UUID.
        type: str
      targetAPs:
        description: Array of target APs.
        elements: str
        type: list
      testMacAddresses:
        description: A string-string test MAC address.
        type: dict
      wiredApplicationMessage:
        description: Wired application message.
        type: str
      wiredApplicationStatus:
        description: Wired application status.
        type: str
      xorSensor:
        description: Is XOR sensor.
        type: bool
    type: list
  showWlcUpgradeBanner:
    description: Show WLC upgrade banner.
    type: bool
  siteHierarchy:
    description: Site hierarchy.
    type: str
  ssids:
    description: Sensor Test Template Edit's ssids.
    elements: dict
    suboptions:
      authProtocol:
        description: Auth protocol.
        type: str
      authType:
        description: Authentication type OPEN, WPA2_PSK, WPA2_EAP, WEB_AUTH, MAB, DOT1X,
          OTHER.
        type: str
      bands:
        description: WIFI bands 2.4GHz or 5GHz.
        type: str
      certdownloadurl:
        description: Certificate download URL.
        type: str
      certfilename:
        description: Auth certificate file name.
        type: str
      certpassphrase:
        description: Certificate password phrase.
        type: str
      certstatus:
        description: Certificate status INACTIVE or ACTIVE.
        type: str
      certxferprotocol:
        description: Certificate transfering protocol HTTP or HTTPS.
        type: str
      eapMethod:
        description: WPA2_EAP methods EAP-FAST, PEAP-MSCHAPv2, EAP-TLS, PEAP-TLS, EAP-TTLS-MSCHAPv2,
          EAP-TTLS-PAP, EAP-TTLS-CHAP, EAP-FAST-GTC, EAP-PEAP-GTC.
        type: str
      extWebAuth:
        description: Indication of using external WEB Auth.
        type: bool
      extWebAuthAccessUrl:
        description: External WEB Auth access URL.
        type: str
      extWebAuthHtmlTag:
        description: Sensor Test Template Edit's extWebAuthHtmlTag.
        elements: dict
        suboptions:
          label:
            description: Label.
            type: str
          tag:
            description: Tag.
            type: str
          value:
            description: Value.
            type: str
        type: list
      extWebAuthPortal:
        description: External authentication portal.
        type: str
      extWebAuthVirtualIp:
        description: External WEB Auth virtual IP.
        type: str
      id:
        description: Identification number.
        type: int
      layer3webAuthEmailAddress:
        description: Layer 3 WEB Auth email address.
        type: str
      layer3webAuthpassword:
        description: Layer 3 WEB Auth password.
        type: str
      layer3webAuthsecurity:
        description: Layer 3 WEB Auth security.
        type: str
      layer3webAuthuserName:
        description: Layer 3 WEB Auth user name.
        type: str
      numAps:
        description: Number of APs in the test.
        type: int
      numSensors:
        description: Number of Sensors in the test.
        type: int
      password:
        description: Password string for onboarding SSID.
        type: str
      passwordType:
        description: SSID password type ASCII or HEX.
        type: str
      profileName:
        description: The SSID profile name string.
        type: str
      proxyPassword:
        description: Proxy server password.
        type: str
      proxyPort:
        description: Proxy server port.
        type: str
      proxyServer:
        description: Proxy server for onboarding SSID.
        type: str
      proxyUserName:
        description: Proxy server user name.
        type: str
      psk:
        description: Password of SSID when passwordType is ASCII.
        type: str
      qosPolicy:
        description: QoS policy PlATINUM, GOLD, SILVER, BRONZE.
        type: str
      scep:
        description: Secure certificate enrollment protocol true or false or null for
          not applicable.
        type: bool
      ssid:
        description: The SSID string.
        type: str
      status:
        description: WLAN status ENABLED or DISABLED.
        type: str
      tests:
        description: Sensor Test Template Edit's tests.
        elements: dict
        suboptions:
          config:
            description: Sensor Test Template Edit's config.
            elements: dict
            suboptions:
              direction:
                description: IPerf direction (UPLOAD, DOWNLOAD, BOTH).
                type: str
              domains:
                description: DNS domain name.
                elements: str
                type: list
              downlinkTest:
                description: Downlink test.
                type: bool
              endPort:
                description: IPerf end port.
                type: int
              exitCommand:
                description: Exit command.
                type: str
              finalPrompt:
                description: Final prompt.
                type: str
              ndtServer:
                description: NDT server.
                type: str
              ndtServerPath:
                description: NDT server path.
                type: str
              ndtServerPort:
                description: NDT server port.
                type: str
              numPackets:
                description: Number of packets.
                type: int
              password:
                description: Password.
                type: str
              passwordPrompt:
                description: Password prompt.
                type: str
              pathToDownload:
                description: File path for file transfer.
                type: str
              port:
                description: Radius or WEB server port.
                type: int
              probeType:
                description: Probe type.
                type: str
              protocol:
                description: Protocol used by file transfer, IPerf, mail server, and
                  radius (TCP, UDP, FTP, POP3, IMAP, CHAP, PAP).
                type: str
              proxyPassword:
                description: Proxy password.
                type: str
              proxyPort:
                description: Proxy port.
                type: str
              proxyServer:
                description: Proxy server.
                type: str
              proxyUserName:
                description: Proxy user name.
                type: str
              server:
                description: Ping, file transfer, mail, radius, ssh, or telnet server.
                type: str
              servers:
                description: IPerf server list.
                elements: str
                type: list
              sharedSecret:
                description: Shared secret.
                type: str
              startPort:
                description: IPerf start port.
                type: int
              transferType:
                description: File transfer type (UPLOAD, DOWNLOAD, BOTH).
                type: str
              udpBandwidth:
                description: IPerf UDP bandwidth.
                type: int
              uplinkTest:
                description: Uplink test.
                type: bool
              url:
                description: URL.
                type: str
              userName:
                description: User name.
                type: str
              userNamePrompt:
                description: User name prompt.
                type: str
            type: list
          name:
            description: Name of the test.
            type: str
        type: list
      thirdParty:
        description: Sensor Test Template Edit's thirdParty.
        suboptions:
          selected:
            description: True the SSID is third party.
            type: bool
        type: dict
      username:
        description: User name string for onboarding SSID.
        type: str
      validFrom:
        description: Valid From UTC timestamp.
        type: int
      validTo:
        description: Valid To UTC timestamp.
        type: int
      whiteList:
        description: Indication of being on allowed list.
        type: bool
      wlanId:
        description: WLAN ID.
        type: int
      wlc:
        description: WLC IP addres.
        type: str
    type: list
  startTime:
    description: Start time.
    type: int
  status:
    description: Status of the test (RUNNING, NOTRUNNING).
    type: str
  templateName:
    description: The test template name that is to be edited.
    type: str
  testScheduleMode:
    description: Test schedule mode (ONDEMAND, DEDICATED, SCHEDULED, CONTINUOUS, RUNNOW).
    type: str
  version:
    description: The sensor test template version (must be 2).
    type: int
  wlans:
    description: WLANs list.
    elements: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Sensors EditSensorTestTemplateV1
  description: Complete reference of the EditSensorTestTemplateV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!edit-sensor-test-template
notes:
  - SDK Method used are
    sensors.Sensors.edit_sensor_test_template_v1,

  - Paths used are
    put /dna/intent/api/v1/AssuranceScheduleSensorTest,
  - It should be noted that this module is an alias of sensor_test_template_edit_v1

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.sensor_test_template_edit:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    _id: string
    actionInProgress: string
    apCoverage:
    - bands: string
      numberOfApsToTest: 0
      rssiThreshold: 0
    connection: string
    encryptionMode: string
    frequency:
      unit: string
      value: 0
    lastModifiedTime: 0
    location: string
    locationInfoList:
    - allSensors: true
      customManagementVlan: true
      locationId: string
      locationType: string
      macAddressList:
      - string
      managementVlan: string
      siteHierarchy: string
    modelVersion: 0
    name: string
    numAssociatedSensor: 0
    numNeighborAPThreshold: 0
    profiles:
    - authProtocol: string
      authType: string
      certdownloadurl: string
      certfilename: string
      certpassphrase: string
      certstatus: string
      certxferprotocol: string
      deviceType: string
      eapMethod: string
      extWebAuth: true
      extWebAuthAccessUrl: string
      extWebAuthHtmlTag:
      - label: string
        tag: string
        value: string
      extWebAuthPortal: string
      extWebAuthVirtualIp: string
      locationVlanList:
      - locationId: string
        vlans:
        - string
      password: string
      passwordType: string
      profileName: string
      psk: string
      qosPolicy: string
      scep: true
      tests:
      - config:
        - direction: string
          domains:
          - string
          downlinkTest: true
          endPort: 0
          exitCommand: string
          finalPrompt: string
          ndtServer: string
          ndtServerPath: string
          ndtServerPort: string
          numPackets: 0
          password: string
          passwordPrompt: string
          pathToDownload: string
          port: 0
          probeType: string
          protocol: string
          proxyPassword: string
          proxyPort: string
          proxyServer: string
          proxyUserName: string
          server: string
          servers:
          - string
          sharedSecret: string
          startPort: 0
          transferType: string
          udpBandwidth: 0
          uplinkTest: true
          url: string
          userName: string
          userNamePrompt: string
        name: string
      username: string
      vlan: string
      whiteList: true
    radioAsSensorRemoved: true
    rssiThreshold: 0
    runNow: string
    scheduleInDays: 0
    sensors:
    - allSensorAddition: true
      assigned: true
      configUpdated: string
      hostName: string
      iPerfInfo: {}
      id: string
      ipAddress: string
      locationId: string
      macAddress: string
      markedForUninstall: true
      name: string
      runNow: string
      sensorType: string
      servicePolicy: string
      status: string
      switchMac: string
      switchSerialNumber: string
      switchUuid: string
      targetAPs:
      - string
      testMacAddresses: {}
      wiredApplicationMessage: string
      wiredApplicationStatus: string
      xorSensor: true
    showWlcUpgradeBanner: true
    siteHierarchy: string
    ssids:
    - authProtocol: string
      authType: string
      bands: string
      certdownloadurl: string
      certfilename: string
      certpassphrase: string
      certstatus: string
      certxferprotocol: string
      eapMethod: string
      extWebAuth: true
      extWebAuthAccessUrl: string
      extWebAuthHtmlTag:
      - label: string
        tag: string
        value: string
      extWebAuthPortal: string
      extWebAuthVirtualIp: string
      id: 0
      layer3webAuthEmailAddress: string
      layer3webAuthpassword: string
      layer3webAuthsecurity: string
      layer3webAuthuserName: string
      numAps: 0
      numSensors: 0
      password: string
      passwordType: string
      profileName: string
      proxyPassword: string
      proxyPort: string
      proxyServer: string
      proxyUserName: string
      psk: string
      qosPolicy: string
      scep: true
      ssid: string
      status: string
      tests:
      - config:
        - direction: string
          domains:
          - string
          downlinkTest: true
          endPort: 0
          exitCommand: string
          finalPrompt: string
          ndtServer: string
          ndtServerPath: string
          ndtServerPort: string
          numPackets: 0
          password: string
          passwordPrompt: string
          pathToDownload: string
          port: 0
          probeType: string
          protocol: string
          proxyPassword: string
          proxyPort: string
          proxyServer: string
          proxyUserName: string
          server: string
          servers:
          - string
          sharedSecret: string
          startPort: 0
          transferType: string
          udpBandwidth: 0
          uplinkTest: true
          url: string
          userName: string
          userNamePrompt: string
        name: string
      thirdParty:
        selected: true
      username: string
      validFrom: 0
      validTo: 0
      whiteList: true
      wlanId: 0
      wlc: string
    startTime: 0
    status: string
    templateName: string
    testScheduleMode: string
    version: 0
    wlans:
    - string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "name": "string",
        "_id": "string",
        "version": 0,
        "modelVersion": 0,
        "startTime": 0,
        "lastModifiedTime": 0,
        "numAssociatedSensor": 0,
        "location": "string",
        "siteHierarchy": "string",
        "status": "string",
        "connection": "string",
        "actionInProgress": "string",
        "frequency": {
          "value": 0,
          "unit": "string"
        },
        "rssiThreshold": 0,
        "numNeighborAPThreshold": 0,
        "scheduleInDays": 0,
        "wlans": [
          "string"
        ],
        "ssids": [
          {
            "bands": "string",
            "ssid": "string",
            "profileName": "string",
            "numAps": 0,
            "numSensors": 0,
            "layer3webAuthsecurity": "string",
            "layer3webAuthuserName": "string",
            "layer3webAuthpassword": "string",
            "layer3webAuthEmailAddress": "string",
            "thirdParty": {
              "selected": true
            },
            "id": 0,
            "wlanId": 0,
            "wlc": "string",
            "validFrom": 0,
            "validTo": 0,
            "status": "string",
            "proxyServer": "string",
            "proxyPort": "string",
            "proxyUserName": "string",
            "proxyPassword": "string",
            "authType": "string",
            "psk": "string",
            "username": "string",
            "password": "string",
            "passwordType": "string",
            "eapMethod": "string",
            "scep": true,
            "authProtocol": "string",
            "certfilename": "string",
            "certxferprotocol": "string",
            "certstatus": "string",
            "certpassphrase": "string",
            "certdownloadurl": "string",
            "extWebAuthVirtualIp": "string",
            "extWebAuth": true,
            "whiteList": true,
            "extWebAuthPortal": "string",
            "extWebAuthAccessUrl": "string",
            "extWebAuthHtmlTag": [
              {
                "label": "string",
                "tag": "string",
                "value": "string"
              }
            ],
            "qosPolicy": "string",
            "tests": [
              {
                "name": "string",
                "config": [
                  {
                    "domains": [
                      "string"
                    ],
                    "server": "string",
                    "userName": "string",
                    "password": "string",
                    "url": "string",
                    "port": 0,
                    "protocol": "string",
                    "servers": [
                      "string"
                    ],
                    "direction": "string",
                    "startPort": 0,
                    "endPort": 0,
                    "udpBandwidth": 0,
                    "probeType": "string",
                    "numPackets": 0,
                    "pathToDownload": "string",
                    "transferType": "string",
                    "sharedSecret": "string",
                    "ndtServer": "string",
                    "ndtServerPort": "string",
                    "ndtServerPath": "string",
                    "uplinkTest": true,
                    "downlinkTest": true,
                    "proxyServer": "string",
                    "proxyPort": "string",
                    "proxyUserName": "string",
                    "proxyPassword": "string",
                    "userNamePrompt": "string",
                    "passwordPrompt": "string",
                    "exitCommand": "string",
                    "finalPrompt": "string"
                  }
                ]
              }
            ]
          }
        ],
        "profiles": [
          {
            "authType": "string",
            "psk": "string",
            "username": "string",
            "password": "string",
            "passwordType": "string",
            "eapMethod": "string",
            "scep": true,
            "authProtocol": "string",
            "certfilename": "string",
            "certxferprotocol": "string",
            "certstatus": "string",
            "certpassphrase": "string",
            "certdownloadurl": "string",
            "extWebAuthVirtualIp": "string",
            "extWebAuth": true,
            "whiteList": true,
            "extWebAuthPortal": "string",
            "extWebAuthAccessUrl": "string",
            "extWebAuthHtmlTag": [
              {
                "label": "string",
                "tag": "string",
                "value": "string"
              }
            ],
            "qosPolicy": "string",
            "tests": [
              {
                "name": "string",
                "config": [
                  {
                    "domains": [
                      "string"
                    ],
                    "server": "string",
                    "userName": "string",
                    "password": "string",
                    "url": "string",
                    "port": 0,
                    "protocol": "string",
                    "servers": [
                      "string"
                    ],
                    "direction": "string",
                    "startPort": 0,
                    "endPort": 0,
                    "udpBandwidth": 0,
                    "probeType": "string",
                    "numPackets": 0,
                    "pathToDownload": "string",
                    "transferType": "string",
                    "sharedSecret": "string",
                    "ndtServer": "string",
                    "ndtServerPort": "string",
                    "ndtServerPath": "string",
                    "uplinkTest": true,
                    "downlinkTest": true,
                    "proxyServer": "string",
                    "proxyPort": "string",
                    "proxyUserName": "string",
                    "proxyPassword": "string",
                    "userNamePrompt": "string",
                    "passwordPrompt": "string",
                    "exitCommand": "string",
                    "finalPrompt": "string"
                  }
                ]
              }
            ],
            "profileName": "string",
            "deviceType": "string",
            "vlan": "string",
            "locationVlanList": [
              {
                "locationId": "string",
                "vlans": [
                  "string"
                ]
              }
            ]
          }
        ],
        "testScheduleMode": "string",
        "showWlcUpgradeBanner": true,
        "radioAsSensorRemoved": true,
        "encryptionMode": "string",
        "runNow": "string",
        "locationInfoList": [
          {
            "locationId": "string",
            "locationType": "string",
            "allSensors": true,
            "siteHierarchy": "string",
            "macAddressList": [
              "string"
            ],
            "managementVlan": "string",
            "customManagementVlan": true
          }
        ],
        "sensors": [
          {
            "name": "string",
            "macAddress": "string",
            "switchMac": "string",
            "switchUuid": "string",
            "switchSerialNumber": "string",
            "markedForUninstall": true,
            "ipAddress": "string",
            "hostName": "string",
            "wiredApplicationStatus": "string",
            "wiredApplicationMessage": "string",
            "assigned": true,
            "status": "string",
            "xorSensor": true,
            "targetAPs": [
              "string"
            ],
            "runNow": "string",
            "locationId": "string",
            "allSensorAddition": true,
            "configUpdated": "string",
            "sensorType": "string",
            "testMacAddresses": {},
            "id": "string",
            "servicePolicy": "string",
            "iPerfInfo": {}
          }
        ],
        "apCoverage": [
          {
            "bands": "string",
            "numberOfApsToTest": 0,
            "rssiThreshold": 0
          }
        ]
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sgt
short_description: Resource module for SGt
description:
- Manage operations create, update and delete of the resource SGt.
- This API creates a security group.
- This API deletes a security group.
- This API allows the client to update a security group.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  defaultSGACLs:
    description: SGt's defaultSGACLs.
    elements: dict
    type: list
  description:
    description: SGt's description.
    type: str
  generationId:
    description: SGt's generationId.
    type: str
  id:
    description: SGt's id.
    type: str
  isReadOnly:
    description: IsReadOnly flag.
    type: bool
  name:
    description: SGt's name.
    type: str
    required: true
  propogateToApic:
    description: PropogateToApic flag.
    type: bool
  value:
    description: Value range 2 ot 65519 or -1 to auto-generate.
    type: int
    required: true
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
seealso:
- name: Cisco ISE documentation for SecurityGroups
  description: Complete reference of the SecurityGroups API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!sgt
notes:
  - SDK Method used are
    security_groups.SecurityGroups.create_security_group,
    security_groups.SecurityGroups.delete_security_group_by_id,
    security_groups.SecurityGroups.update_security_group_by_id,

  - Paths used are
    post /ers/config/sgt,
    delete /ers/config/sgt/{id},
    put /ers/config/sgt/{id},

"""

EXAMPLES = r"""
- name: Update by id
  cisco.ise.sgt:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    defaultSGACLs:
    - {}
    description: string
    generationId: string
    id: string
    isReadOnly: true
    name: string
    propogateToApic: true
    value: 0

- name: Delete by id
  cisco.ise.sgt:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: absent
    id: string

- name: Create
  cisco.ise.sgt:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    defaultSGACLs:
    - {}
    description: string
    generationId: string
    isReadOnly: true
    name: string
    propogateToApic: true
    value: 0

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "name": "string",
      "description": "string",
      "value": 0,
      "generationId": "string",
      "isReadOnly": true,
      "propogateToApic": true,
      "defaultSGACLs": [
        {}
      ],
      "link": {
        "rel": "string",
        "href": "string",
        "type": "string"
      }
    }

ise_update_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  version_added: '1.1.0'
  type: dict
  sample: >
    {
      "UpdatedFieldsList": {
        "updatedField": [
          {
            "field": "string",
            "oldValue": "string",
            "newValue": "string"
          }
        ],
        "field": "string",
        "oldValue": "string",
        "newValue": "string"
      }
    }
"""
